// BasicAgent.cpp: implementation of the CBasicAgent class
//
///////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Goalie.h"
#include "BasicAgent.h"

#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif	// _DEBUG

// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CBasicAgent, CSocketAgent)
	//{{AFX_MSG_MAP(CBasicAgent)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

///////////////////////////////////////////////////////////////////////////////
// CBasicAgent construction

CBasicAgent::CBasicAgent()
{
	m_bGoalie = false;
	m_bConnected = false;
	m_bDebug = false;
	m_strFileName = "debug.log";
}

///////////////////////////////////////////////////////////////////////////////
// CBasicAgent operations

bool CBasicAgent::ConnectAgent(LPCTSTR psTeam, LPCTSTR psServer, unsigned uPort, bool bGoalie)
{
	// is the agent already connected (should not happen)
	if (m_bConnected) return true;

	// create the datagram socket
	if (!Create(uPort, psServer)) {
		// socket could not be created
		OnErrorCreatingSocket();
		return false;
	}
	m_bConnected = true;
	m_strTeam = psTeam;
	m_bGoalie = bGoalie;
	m_perception.SetTeam(m_strTeam);
	OnCreatedSocket();

	if (m_bDebug) m_pFile = fopen(m_strFileName, "w");
	m_perception.SetDebug(m_bDebug, m_pFile);

	return true;
}

void CBasicAgent::DisconnectAgent()
{
	char Buffer[50];

	// is the socket active ?
	if (m_bConnected) {
		m_perception.SetDebug(false, NULL);
		if (m_bDebug) fclose(m_pFile);
		Bye();
		ShutDown();
		while(Receive(Buffer,50) > 0);
		Close();
	}
	m_bConnected = false;
}

void CBasicAgent::Move(double dX, double dY)
{
	CString	msg;

	msg.Format("(move %.2f %.2f)", dX, dY);
	SendMsg(msg);
}

void CBasicAgent::Dash(double dPower)
{
	CString	msg;

	// power may be -30 to 100
	msg.Format("(dash %.2f)", dPower);
	SendMsg(msg);
}

void CBasicAgent::Kick(double dPower, double dDir)
{
	CString	msg;

	// power -30 to 100
	// direction -180 to 180
	msg.Format("(kick %.2f %.2f)", dPower, dDir);
	SendMsg(msg);
}

void CBasicAgent::Turn(double dMoment)
{
	CString	msg;

	// angle may be -180 to 180
	msg.Format("(turn %.2f)", dMoment);
	SendMsg(msg);
}

void CBasicAgent::TurnNeck(double dAng)
{
	CString	msg;

	// angle may be -180 to 180
	// resulting angle may be -90 to 90
	msg.Format("(turn_neck %.2f)", dAng);
	SendMsg(msg);
}

void CBasicAgent::Catch(double dDir)
{
	CString	msg;

	// direction -180 to 180
	msg.Format("(catch %.2f)", dDir);
	SendMsg(msg);
}

void CBasicAgent::Say(const CString& strMsg)
{
	CString	msg("(say " + strMsg + ")");

	// max 255 byte
//	msg.Format("(say ");
//	msg += strMsg;
//	msg += _T(")");
	SendMsg(msg);
}

void CBasicAgent::ChangeView(int iWidth, int iQuality)
{
	CString	msg("(change_view ");

	// set the view angle string
	if (iWidth == 180) msg += _T("wide ");
	else if (iWidth == 90) msg += _T("normal ");
	else msg += _T("narrow ");
	// set the view quality string
	if (iQuality == 0) msg += _T("low)");
	else msg += _T("high)");

	SendMsg(msg);
	OnChangedView(iWidth, iQuality);
}

void CBasicAgent::Bye()
{
	CString	msg("(bye)");

	SendMsg(msg);
}

void CBasicAgent::SendMsg(const CString& strMsg)
{
	int iLength;

	// cannot send anything when not connected
	if (!m_bConnected) return;
	// send the buffer
	iLength = SendData(strMsg, strMsg.GetLength());
	if (iLength == strMsg.GetLength()) OnSentMessage((LPCTSTR)strMsg);
	else {
		// an error occured
		#ifdef _DEBUG
			afxDump << "error writing to socket\n";
		#endif
		OnErrorSendingMessage();
	}
}

///////////////////////////////////////////////////////////////////////////////
// CBasicAgent overrides

void CBasicAgent::Play()
{
	// this is the main function for actions of the agent
	// override to provide player functionality
}

void CBasicAgent::OnCreatedSocket()
{
	CString	msg;

	// send the (init team) command
	if (m_bGoalie) msg.Format("(init %s (player) (version 5.00))", (LPCTSTR)m_strTeam);
	else msg.Format("(init %s (version 5.00))", (LPCTSTR)m_strTeam);

	SendMsg(msg);
}

void CBasicAgent::OnSentMessage(LPCTSTR psMsg)
{
	CString str(psMsg);

	if (str.Find("turn_neck") == -1) {
		m_strPriMsg = str;
		m_bPriMsg = true;
	}
	else {
		m_strSecMsg = str;
		m_bSecMsg = true;
	}
}

void CBasicAgent::OnReceivedMessage(LPCTSTR psMsg)
{
	int iTime = m_perception.GetSimTime();
	int iSeeTime = m_perception.GetSeeTime();
	int iPlayMode = m_perception.GetPlayMode();
	CParseString str(psMsg);
	CString msgType;

	str = str.ParseBracket();
	msgType = str.ParseString();

	if (m_bDebug) {
		fprintf(m_pFile, "\nTime = %d\n", iTime);
		fprintf(m_pFile, "MsgType = %s\n", msgType);
	}

	if (iTime < str.ParseInteger() || iPlayMode == PM_BEFORE_KICK_OFF ||
		iPlayMode == PM_GOAL_L || iPlayMode == PM_GOAL_R)
	{
		if (m_bPriMsg) {
			if (m_bDebug) fprintf(m_pFile, "Updating agent : %s\n", m_strPriMsg);
			m_perception.UpdateAgent(m_strPriMsg);
			m_bPriMsg = false;
		}
		if (m_bSecMsg) {
			if (m_bDebug) fprintf(m_pFile, "Updating agent : %s\n", m_strSecMsg);
			m_perception.UpdateAgent(m_strSecMsg);
			m_bSecMsg = false;
		}
		if (m_bDebug) fprintf(m_pFile, "Updating field\n");
		m_perception.UpdateField();
	}

	m_perception.ParseMessage(psMsg);
	if (m_bDebug) fprintf(m_pFile, "NewTime = %d\n", m_perception.GetSimTime());

	if (m_perception.GetViewAngle() == 45) {
		if (msgType == "see" && iSeeTime < m_perception.GetSeeTime()) Play();
		return;
	}
	iPlayMode = m_perception.GetPlayMode();
	if (iTime < m_perception.GetSimTime() || iPlayMode == PM_BEFORE_KICK_OFF ||
		iPlayMode == PM_GOAL_L || iPlayMode == PM_GOAL_R) Play();
}

void CBasicAgent::OnReceive(int nErrorCode)
{
	char buffer[MAX_MSG_LENGTH];
	int iLength = 0;

	// first call base classes method
	CSocketAgent::OnReceive(nErrorCode);

	// read a message from the socket
	iLength = ReceiveData(buffer, MAX_MSG_LENGTH);

	if (iLength > 0) OnReceivedMessage((LPCTSTR)buffer);
	else {
		// an error occured
		#ifdef _DEBUG
			afxDump << "error reading socket\n";
		#endif
		OnErrorReceivingMessage();
	}
}

///////////////////////////////////////////////////////////////////////////////
// CBasicAgent implementation

CBasicAgent::~CBasicAgent()
{
	if (m_bConnected) DisconnectAgent();
}
