// BasicAgent.h: interface for the CBasicAgent class
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __AFX_BASICAGENT_H_
#define __AFX_BASICAGENT_H_

#if _MSC_VER > 1000
#pragma once
#endif	// _MSC_VER > 1000

// BasicAgent.h header file declares following classes

class CBasicAgent;

#include "SocketAgent.h"
#include "Perception.h"

#define MAX_MSG_LENGTH 1500

///////////////////////////////////////////////////////////////////////////////
// Class: CBasicAgent
// Date: 15.04.01
// Version Date: 11.05.01
// Version: 1.11
// Author: Marek Lehuta
//
// Base Class: CSocketAgent
///////////////////////////////////////////////////////////////////////////////
// Description: provides basic functionality for all agents
///////////////////////////////////////////////////////////////////////////////

class CBasicAgent : public CSocketAgent
{
// Construction
public:
	CBasicAgent();

// Attributes
protected:
	bool m_bGoalie;
	bool m_bConnected;			// true, if the agent is currently connected
	CPerception m_perception;
	CString m_strTeam;			// name of the team the player is in

	CString m_strPriMsg;		// last primary message sent to the server
	bool m_bPriMsg;				// true, if primary message was sent to the server last cycle
	CString m_strSecMsg;		// last secondary message sent to the server
	bool m_bSecMsg;				// true, if secondary message was sent to the server last cycle

	bool m_bDebug;				// true, if debug output is switched on
	CString m_strFileName;		// debug file name
	FILE* m_pFile;				// debug file structure

// Operations
public:
	bool ConnectAgent(LPCTSTR psTeam, LPCTSTR psServer, unsigned uPort, bool bGoalie);
	void DisconnectAgent();

	// attribute fuctions
	bool IsConnected() const { return m_bConnected; };
	const CPerception& GetPerception() const { return m_perception; };

	// the agents actions
	void Move(double dX, double dY);
	void Dash(double dPower);
	void Kick(double dPower, double dDir);
	void Turn(double dMoment);
	void TurnNeck(double dAng);
	void Catch(double dDir);
	void Say(const CString& strMsg);
	void ChangeView(int iWidth, int iQuality);
	void Bye();

	// communication functions
	void SendMsg(const CString& strMsg);

// Overrides
public:
	// main playing method
	virtual void Play();

	virtual void OnCreatedSocket();
	virtual void OnSentMessage(LPCTSTR psMsg);
	virtual void OnReceivedMessage(LPCTSTR psMsg);
	virtual void OnChangedView(int iWidth, int iQuality) {};

	virtual void OnErrorCreatingSocket() {};
	virtual void OnErrorSendingMessage() {};
	virtual void OnErrorReceivingMessage() {};

	// ClassWizard generated virtual function overrides
public:
	//{{AFX_VIRTUAL(CBasicAgent)
	virtual void OnReceive(int nErrorCode);
	//}}AFX_VIRTUAL

// Message handlers
	// Generated message map functions
public:
	//{{AFX_MSG(CBasicAgent)
		// NOTE - the ClassWizard will add and remove member functions here.
	//}}AFX_MSG

// Implementation
public:
	virtual ~CBasicAgent();

};

///////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif	// __AFX_BASICAGENT_H_
