// ComplexAgent.cpp: implementation of the CComplexAgent class
//
///////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Goalie.h"
#include "ComplexAgent.h"

#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif	// _DEBUG

// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CComplexAgent, CBasicAgent)
	//{{AFX_MSG_MAP(CComplexAgent)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

///////////////////////////////////////////////////////////////////////////////
// CComplexAgent construction

CComplexAgent::CComplexAgent()
{
}

///////////////////////////////////////////////////////////////////////////////
// CComplexAgent operations

bool CComplexAgent::MoveAbs(const CPosition& pos, double dTol)
{
	if (fabs(m_perception.GetAgentPlayer()->GetDistanceTo(pos)) > dTol) {
		if (m_perception.GetSide() == SIDE_RIGHT) Move(-pos.GetX(), -pos.GetY());
		else Move(pos.GetX(), pos.GetY());
		return true;
	}
	return false;
}

void CComplexAgent::TurnRel(double dMoment)
{
	double dTurn = dMoment * (1 + INERTIA_MOMENT * m_perception.GetAgentPlayer()->GetSpeedDist());

	if (dTurn > 180) dTurn = 180;
	else if (dTurn < -180) dTurn = -180;
	Turn(dTurn);
}

bool CComplexAgent::TurnToBall(double dTol)
{
	double dBallDir = m_perception.GetBallDirection();

	if (m_perception.GetBall()->IsValid()) {
		if (fabs(dBallDir) > dTol) {
			TurnRel(dBallDir);
			return true;
		}
		return false;
	}
	return false;
}

bool CComplexAgent::TurnToGoal(bool bOur, double dTol)
{
	double dGoalDir = m_perception.GetGoalDirection(NULL, bOur);

	if (fabs(dGoalDir) > dTol) {
		TurnRel(dGoalDir);
		return true;
	}
	return false;
}

bool CComplexAgent::TurnTo(const CPosition& pos, double dTol)
{
	double dPosDir = m_perception.GetAgentPlayer()->GetAngleTo(pos);

	if (fabs(dPosDir) > dTol) {
		TurnRel(dPosDir);
		return true;
	}
	return false;
}

bool CComplexAgent::LookAhead(double dTol)
{
	double dHeadDir = m_perception.GetAgentPlayer()->GetHeadDir();

	if (fabs(dHeadDir) > dTol) {
		TurnNeck(-dHeadDir);
		return true;
	}
	return false;
}

bool CComplexAgent::LookAtBall(double dTol)
{
	double dBallDir = m_perception.GetAgentPlayer()->GetHeadAngleTo(m_perception.GetBall());

	if (m_perception.GetBall()->IsValid()) {
		if (fabs(dBallDir) > dTol) {
			TurnNeck(dBallDir);
			return true;
		}
		return false;
	}
	return LookAhead(dTol);
}

bool CComplexAgent::LookAtGoal(bool bOur, double dTol)
{
	double dGoalDir = m_perception.GetAgentPlayer()->GetHeadAngleTo(m_perception.GetGoal(bOur));

	if (fabs(dGoalDir) > dTol) {
		TurnNeck(dGoalDir);
		return true;
	}
	return false;
}

bool CComplexAgent::LookAt(const CPosition& pos, double dTol)
{
	double dPosDir = m_perception.GetAgentPlayer()->GetHeadAngleTo(pos);

	if (fabs(dPosDir) > dTol) {
		TurnNeck(dPosDir);
		return true;
	}
	return false;
}

bool CComplexAgent::RunTo(const CPosition& destPos, bool bBackw, double dPosTol, double dTurnTol)
{
	CPlayer* pPlayer = m_perception.GetAgentPlayer();
	double dDist = pPlayer->GetDistanceTo(destPos);
	double dAvoidDir = m_perception.GetAvoidAngle(destPos);
//	double dMinDir;

	if (dDist < dPosTol) return false;	// at the position already

	if (dDist > 2) dTurnTol /= 2;
	if (dDist > 10) dTurnTol /= 2;
	if (dDist > 20) dTurnTol /= 2;
//	else dMinDir = (int)(0.5 + asin(0.7 / dDist) * RAD2DEG);

	if (bBackw) {
		// is it acceptable angle to run backwards
		if (fabs(dAvoidDir) > 120) {
			CVector vect(1.0, dAvoidDir + 180.0);
			if (fabs(vect.GetAngle()) > dTurnTol) TurnRel(vect.GetAngle());
			else Dash(-30);
			return true;
		}
	}

	if (fabs(dAvoidDir) > dTurnTol) TurnRel(dAvoidDir);
	else Dash(100);

	return true;
}

bool CComplexAgent::RunToLookingAt(const CPosition& destPos, const CPosition& lookPos, bool bBackw,
	double dPosTol, double dTurnTol, double dLookTol)
{
	CPlayer* pPlayer = m_perception.GetAgentPlayer();
	double dDist = pPlayer->GetDistanceTo(destPos);
	double dLookDir = pPlayer->GetAngleTo(lookPos);
	double dAvoidDir = m_perception.GetAvoidAngle(destPos);
//	double dMinDir;

	if (dDist < dPosTol) return false;	// at the position already

	if (dDist > 2) dTurnTol /= 2;
	if (dDist > 10) dTurnTol /= 2;
	if (dDist > 20) dTurnTol /= 2;
//	else dMinDir = (int)(0.5 + asin(0.7 / dDist) * RAD2DEG);

	if (bBackw) {
		CVector vect(1.0, dAvoidDir - dLookDir);
		// is it acceptable angle to run backwards
		if (fabs(vect.GetAngle()) > 120) {
			vect.SetAngle(dAvoidDir + 180.0);
			if (fabs(vect.GetAngle()) > dTurnTol) TurnRel(vect.GetAngle());
			else Dash(-30);
			LookAt(lookPos, dLookTol);
			return true;
		}
	}

	if (fabs(dAvoidDir) > dTurnTol) TurnRel(dAvoidDir);
	else Dash(100);
	LookAt(lookPos, dLookTol);

	return true;
}

///////////////////////////////////////////////////////////////////////////////
// CComplexAgent overrides

void CComplexAgent::Play()
{
	// this is the main function for actions of the agent
	// override to provide player functionality
	CBasicAgent::Play();
}

///////////////////////////////////////////////////////////////////////////////
// CComplexAgent implementation

CComplexAgent::~CComplexAgent()
{
}
