// Goalie.h: interface of the CGoalieApp class
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __AFX_GOALIE_H_
#define __AFX_GOALIE_H_

#if _MSC_VER > 1000
#pragma once
#endif	// _MSC_VER > 1000

#ifndef __AFXWIN_H__
	#error include 'StdAfx.h' before including this file for PCH
#endif	// __AFXWIN_H__

///////////////////////////////////////////////////////////////////////////////
// Project: SoccerPlayer (SoccerGoalie)
// Date: 11.05.01
// Version: 1.23
///////////////////////////////////////////////////////////////////////////////
// Description: project implementing soccer player (goalie) by the use of
//				methods of artificial intelligence
///////////////////////////////////////////////////////////////////////////////

// Goalie.h header file declares following classes

class CGoalieApp;

#include "Resource.h"			// main symbols

#define PLAYER_MAX_AGE		5
#define BALL_MAX_AGE		10

#define MAX_STAMINA			3500
#define STAMINA_INCREMENT	35
#define PLAYER_SPEED_MAX	1.0
#define PLAYER_DECAY		0.4
#define PLAYER_SIZE			0.3
#define KICKABLE_MARGIN		0.7
#define KICK_POWER_RATE		0.016
#define DASH_POWER_RATE		0.006
#define MAXPOWER			100.0
#define INERTIA_MOMENT		5.0

#define BALL_SPEED_MAX		2.7
#define BALL_DECAY			0.94
#define BALL_SIZE			0.085
#define SEND_STEP			150
#define EFFORT_MIN			0.6
#define CATCHABLE_AREA_L	2.0
#define CATCHABLE_AREA_W	1.0
/*
#define LOW_STAMINA			1000		// point at which it costs effort
*/
#define HALF_TIME 3000
#define GOAL_WIDTH			14.02

#define DEFAULT_TEAM		"Dragons"		// name of default team
#define DEFAULT_SERVER		"localhost"		// server address (name)
#define DEFAULT_PORT		6000			// server socket port
//#define DEFAULT_MASTERCH	6001			// channel for master mode

/****************************** COLORS *******************************/

#define COLOR_LEFT_TEAM RGB(200, 200, 20)		// yellow
#define COLOR_RIGHT_TEAM RGB(200, 20, 20)		//	red
#define COLOR_AGENT RGB(20, 200, 20)			// green
#define COLOR_BALL RGB(20, 20, 230)				// blue
#define COLOR_GOAL RGB(0, 0, 0)					// black
#define COLOR_FLAG RGB(200, 200, 200)			// light gray
#define COLOR_LINE RGB(255, 255, 255)			// white
#define COLOR_ANY_OBJECT RGB(40, 40, 40)		// dark gray
#define COLOR_FIELD RGB(10, 100, 10)			// green
#define COLOR_BACK RGB(20, 150, 20)				// green, a little lighter than above
#define COLOR_SKY RGB(30, 30, 150);				// light blue

/****************************** FLAGS *******************************/
#define SIZE_X			120
#define SIZE_Y			80

#define	GW				7.01		// goal width/2
#define FW				52.5		// field width
#define	FH				34.0		// field height
#define	PW				36.0		// penalty width
#define	PH				20.16		// penalty height

#define FLAG_COUNT		55		//

#define LT_GOAL_FLAG	0
#define LB_GOAL_FLAG	1
#define L_GOAL			2
#define RT_GOAL_FLAG	3
#define RB_GOAL_FLAG	4
#define R_GOAL			5

#define LT_PEN_FLAG		6
#define LB_PEN_FLAG		7
#define LC_PEN_FLAG		8
#define RT_PEN_FLAG		9
#define RB_PEN_FLAG		10
#define RC_PEN_FLAG		11

#define LT_FLAG			12
#define LB_FLAG			16
#define RT_FLAG			21
#define RB_FLAG			25

#define CT_FLAG			52
#define CB_FLAG			53
#define C_FLAG			54

#define FLAG_POSITIONS_X {	-FW, -FW, -FW, FW, FW, FW,\
							-PW, -PW, -PW, PW, PW, PW,\
							-FW, -FW-5, -FW-5, -FW-5, -FW, -FW-5, -FW-5, -FW-5, -FW-5,\
							FW, FW+5, FW+5, FW+5, FW, FW+5, FW+5, FW+5, FW+5,\
							-10, -20, -30, -40, -50, 10, 20, 30, 40, 50, 0,\
							-10, -20, -30, -40, -50, 10, 20, 30, 40, 50, 0,\
							0, 0, 0 }

#define FLAG_POSITIONS_Y {	-GW, GW, 0, -GW, GW, 0,\
							-PH, PH, 0, -PH, PH, 0,\
							-FH, -10, -20, -30, FH, 10, 20, 30, 0,\
							-FH, -10, -20, -30, FH, 10, 20, 30, 0,\
							-FH-5, -FH-5, -FH-5, -FH-5, -FH-5, -FH-5, -FH-5, -FH-5, -FH-5, -FH-5, -FH-5,\
							FH+5, FH+5, FH+5, FH+5, FH+5, FH+5, FH+5, FH+5, FH+5, FH+5, FH+5,\
							-FH, FH, 0 }

/*
#define DIS_VERSION	1					// version info for serialization
#define D_ZERO_P		0.0000001
#define D_ZERO_N		-0.0000001

#define FIELD_FLAG			1
#define PENALTY_AREA_FLAG	2

#define LEFT_TO_RIGHT	0
#define RIGHT_TO_LEFT	1

#define MAX_PLAYERS 22				// maximum number of players
#define PLAYERS_PER_TEAM 11			// actual number of players per team

#define MAX_NUM_PLAYER 50			// max pocet registrovanych hracov
#define MAX_PERCEPTION_LEN 1024	// maximum length of perception message
#define MAX_SAY_LEN					// maximum length of a say statement

#define NOT_EXISTING -32000	// items not existing in a perception
*/
/* playmodes (Left are always odd numbers)
#define PLAY_ON				20
#define HALF_TIME				21
#define TIME_UP				22
#define EXTEND					23
#define FOUL_L					50
#define FOUL_R					60
*/
/*
// for visualizing
#define PLAYERSIZE	6			// radius of the circle representing the player
#define BALLSIZE		4
#define FLAG_SIZE		3
*/

#define PLAYMODE_COUNT		21

#define PLAYMODE_STRINGS {	"",\
							"before_kick_off",\
							"kick_off_l",\
							"kick_off_r",\
							"kick_in_l",\
							"kick_in_r",\
							"free_kick_l",\
							"free_kick_r",\
							"corner_kick_l",\
							"corner_kick_r",\
							"goal_kick_l",\
							"goal_kick_r",\
							"goal_l",\
							"goal_r",\
							"goalie_catch_ball_l",\
							"goalie_catch_ball_r",\
							"play_on",\
							"half_time",\
							"time_up",\
							"time_over",\
							"drop_ball" }

#define PM_UNKNOWN			0
#define PM_BEFORE_KICK_OFF	1
#define PM_KICK_OFF_L		2
#define PM_KICK_OFF_R		3
#define PM_KICK_IN_L		4
#define PM_KICK_IN_R		5
#define PM_FREE_KICK_L		6
#define PM_FREE_KICK_R		7
#define PM_CORNER_KICK_L	8
#define PM_CORNER_KICK_R	9
#define PM_GOAL_KICK_L		10
#define PM_GOAL_KICK_R		11
#define PM_GOAL_L			12
#define PM_GOAL_R			13
#define PM_GOALIE_CATCH_BALL_L	14
#define PM_GOALIE_CATCH_BALL_R	15
#define PM_PLAY_ON			16
#define PM_HALF_TIME		17
#define PM_TIME_UP			18
#define PM_TIME_OVER		19
#define PM_DROP_BALL		20

///////////////////////////////////////////////////////////////////////////////
// Class: CGoalieApp
// Date: 12.04.01
// Version Date: 12.04.01
// Version: 1.00
// Author: Marek Lehuta
//
// Base Class: CWinApp
///////////////////////////////////////////////////////////////////////////////
// Description: main header file for the Goalie application
//				see Goalie.cpp for the implementation of this class
///////////////////////////////////////////////////////////////////////////////

class CGoalieApp : public CWinApp
{
// Construction
public:
	CGoalieApp();

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
public:
	//{{AFX_VIRTUAL(CGoalieApp)
	virtual BOOL InitInstance();
	//}}AFX_VIRTUAL

// Message handlers
	// Generated message map functions
public:
	//{{AFX_MSG(CGoalieApp)
	afx_msg void OnAppAbout();
		// NOTE - the ClassWizard will add and remove member functions here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

// Implementation
public:

};

///////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif	// __AFX_GOALIE_H_
