// GoalieAgent.cpp: implementation of the CGoalieAgent class
//
///////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Goalie.h"
#include "GoalieAgent.h"

#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif	// _DEBUG

extern double gFlagXPos[FLAG_COUNT];
extern double gFlagYPos[FLAG_COUNT];

// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CGoalieAgent, CComplexAgent)
	//{{AFX_MSG_MAP(CGoalieAgent)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

///////////////////////////////////////////////////////////////////////////////
// CGoalieAgent construction

CGoalieAgent::CGoalieAgent()
{
	m_bInPosition = false;
	m_bCaughtBall = false;
	m_bDebug = false;
	m_strFileName = "debug.goalie.log";
}

///////////////////////////////////////////////////////////////////////////////
// CGoalieAgent operations

CPosition CGoalieAgent::GetDefensePos() const
{
	CPosition defPos(m_perception.GetSide() == SIDE_LEFT ? -STD_POS_X : STD_POS_X, STD_POS_Y);
	CPosition ballPos = m_perception.GetBall()->GetAbsPos();
	CPosition tGoal(m_perception.GetSide() == SIDE_LEFT ? gFlagXPos[LT_GOAL_FLAG] : gFlagXPos[RT_GOAL_FLAG], gFlagYPos[LT_GOAL_FLAG]);
	CPosition bGoal(m_perception.GetSide() == SIDE_LEFT ? gFlagXPos[LB_GOAL_FLAG] : gFlagXPos[RB_GOAL_FLAG], gFlagYPos[LB_GOAL_FLAG]);
	CVector tVect = tGoal - ballPos;
	CVector bVect = bGoal - ballPos;
	double dScale;

	tVect.SetAngle(tVect.GetAngle() - bVect.GetAngle());
	bVect.Rotate(tVect.GetAngle() / 2);
	dScale = (defPos - ballPos).GetX() / bVect.GetX();
	bVect.ScaleXY(dScale, dScale);
	ballPos += bVect;
	if (ballPos.GetY() < gFlagYPos[LT_GOAL_FLAG]) ballPos.SetY(gFlagYPos[LT_GOAL_FLAG]);
	else if (ballPos.GetY() > gFlagYPos[LB_GOAL_FLAG]) ballPos.SetY(gFlagYPos[LB_GOAL_FLAG]);
	defPos.SetY(ballPos.GetY());

	return defPos;
}

CPosition CGoalieAgent::GetKickOffPos() const
{
	CPosition kickOffPos(m_perception.GetSide() == SIDE_LEFT ? -STD_POS_X + 5 : STD_POS_X - 5, STD_POS_Y);
	CPlayer* pPlayer;
	double dOppDist = 0.0;
	double dY = 0.0;
	double dDist;

	for (int i = (int)gFlagYPos[LT_GOAL_FLAG] - 5; i <= (int)gFlagYPos[LB_GOAL_FLAG] + 5; i += 2) {
		kickOffPos.SetY(i);
		pPlayer = m_perception.GetNearestOpponentTo(kickOffPos);
		if (pPlayer != NULL) dDist = pPlayer->GetDistanceTo(kickOffPos);
		else dDist = 50.0;
		if (dDist > dOppDist) {
			dOppDist = dDist;
			dY = kickOffPos.GetY();
		}
	}
	kickOffPos.SetY(dY);

	return kickOffPos;
}

///////////////////////////////////////////////////////////////////////////////
// CGoalieAgent overrides

void CGoalieAgent::Play()
{
	// this is the main function for actions of the agent
	// to make the agent play, add your code here

	// call base class method
	CComplexAgent::Play();

	switch (m_perception.GetPlayMode()) {
		case PM_BEFORE_KICK_OFF:
		case PM_GOAL_L:
		case PM_GOAL_R:
			if (!m_bInPosition) {
				Move(-STD_POS_X, STD_POS_Y);
				m_bInPosition = true;
				break;
			}
			if (!m_perception.GetBall()->IsValid()) {
				Turn(45.0);
				break;
			}
			TurnToBall();
			break;
		case PM_PLAY_ON:
		case PM_KICK_OFF_L:
		case PM_KICK_OFF_R:
		case PM_KICK_IN_L:
		case PM_KICK_IN_R:
		case PM_CORNER_KICK_L:
		case PM_CORNER_KICK_R:
		case PM_GOAL_KICK_L:
		case PM_GOAL_KICK_R:
			if (m_bInPosition) m_bInPosition = false;

			if (m_bDebug) {
				CPlayer* pPlayer = m_perception.GetAgentPlayer();
				CBall* pBall = m_perception.GetBall();
				fprintf(m_pFile, "PlayerPos = %.2f,%.2f\n", pPlayer->GetAbsX(), pPlayer->GetAbsY());
				fprintf(m_pFile, "PlayerSpeed = %.2f, Dir = %.2f\n", pPlayer->GetSpeedDist(), pPlayer->GetBodyDir());
				fprintf(m_pFile, "PlayerHead = %.2f\n", pPlayer->GetHeadDir());
				fprintf(m_pFile, "BallPos = %.2f,%.2f\n", pBall->GetAbsX(), pBall->GetAbsY());
				fprintf(m_pFile, "BallSpeed = %.2f, Dir = %.2f\n", pBall->GetSpeedDist(), pBall->GetSpeedDir());
				fprintf(m_pFile, "BallDist = %.2f, Dir = %.2f\n", pBall->GetDistance(), pBall->GetDirection());
			}

			if (!m_perception.GetBall()->IsValid()) {
				Turn(45.0);
				LookAhead(5.0);
				break;
			}
//			if (m_perception.GetBall()->GetDistanceTo(m_perception.GetGoal(true)) > 50.0 ||
			if (!m_perception.IsOnOurHalf(m_perception.GetBall()) ||
				m_perception.GetPlayMode() == PM_CORNER_KICK_L || m_perception.GetPlayMode() == PM_CORNER_KICK_R ||
				m_perception.GetPlayMode() == PM_GOAL_KICK_L || m_perception.GetPlayMode() == PM_GOAL_KICK_R)
			{
				CPosition stdPos(m_perception.GetSide() == SIDE_LEFT ? -STD_POS_X : STD_POS_X, STD_POS_Y);
//				if (RunTo(stdPos, true, 2.0)) {
				if (RunToLookingAt(stdPos, m_perception.GetBall()->GetAbsPos(), true, 2.0)) {
//					LookAtBall();
					break;
				}
				TurnToBall();
				LookAtBall();
				break;
			}

			if (m_bDebug) {
				CPlayer* pPlayer = m_perception.GetAgentPlayer();
				fprintf(m_pFile, "Nearest = %d, %d; Agent = %d, %d\n", m_perception.GetBallNearestPlayer()->GetSide(),
					m_perception.GetBallNearestPlayer()->GetNumber(), pPlayer->GetSide(), pPlayer->GetNumber());
				fprintf(m_pFile, "BallDist = %.2f\n", (m_perception.GetBallInterceptPos() - m_perception.GetGoal(true)->GetAbsPos()).GetLength());
			}

//			if (m_perception.GetBall()->GetDistanceTo(m_perception.GetGoal(true)) > 20.0) {
			if ((m_perception.GetBallNearestPlayer() != m_perception.GetAgentPlayer()) ||
				((m_perception.GetBallInterceptPos() - m_perception.GetGoal(true)->GetAbsPos()).GetLength() > 25.0))
			{
				m_bCaughtBall = false;
				CPosition defPos = GetDefensePos();
//				if (RunTo(defPos, true)) {
				if (RunToLookingAt(defPos, m_perception.GetBall()->GetAbsPos(), true)) {
//					LookAtBall();
					break;
				}
				TurnToBall(10.0);
				LookAtBall(5.0);
				break;
			}
			if (m_bCaughtBall) {
/*				if (m_perception.GetBallDistance() > KICKABLE_MARGIN) {
					Dash(100.0);
					break;
				}*/
				if (m_perception.GetBallDistance() < KICKABLE_MARGIN) {
					Kick(100.0, m_perception.GetAvoidAngle(m_perception.GetGoal()->GetAbsPos()));
					ChangeView(90, 1);
					m_bCaughtBall = false;
					break;
				}
				if (RunTo(m_perception.GetBallInterceptPos(), false, PLAYER_SIZE)) break;
//				Kick(100.0, m_perception.GetAvoidAngle(m_perception.GetGoal()->GetAbsPos()));
//				m_bCaughtBall = false;
				break;
			}
			if (m_perception.GetBallDistance() < CATCHABLE_AREA_W) {
				if (m_perception.IsInPenaltyArea(m_perception.GetBall())) {
					Catch(m_perception.GetBallDirection());
					break;
				}
				if (m_perception.GetBallDistance() < KICKABLE_MARGIN) {
					Kick(100.0, m_perception.GetAvoidAngle(m_perception.GetGoal()->GetAbsPos()));
				}
				else {
					RunTo(m_perception.GetBallInterceptPos(), false, PLAYER_SIZE, 10.0);
					LookAtBall(5.0);
				}
				break;
			}
			RunTo(m_perception.GetBallInterceptPos(), false, KICKABLE_MARGIN, 10.0);
			LookAtBall(5.0);
			break;
		case PM_FREE_KICK_L:
			if (m_perception.GetSide() != SIDE_LEFT) break;
		case PM_FREE_KICK_R:
			if (m_perception.GetPlayMode() == PM_FREE_KICK_R && m_perception.GetSide() != SIDE_RIGHT) break;
			if (!m_bCaughtBall) {
				m_bCaughtBall = true;
				LookAhead();
				ChangeView(45, 1);
				break;
			}
			if (TurnToGoal(false, 10.0)) break;
			if (!m_bInPosition) {
				CPosition kickOffPos = GetKickOffPos();
				MoveAbs(kickOffPos);
				m_bInPosition = true;
				break;
			}
//			Kick(50.0, m_perception.GetGoalDirection());
			Kick(50.0, 0);
			break;
//		case PM_TIME_OVER:
		default:;
	}
}

///////////////////////////////////////////////////////////////////////////////
// CGoalieAgent implementation

CGoalieAgent::~CGoalieAgent()
{
}
