// GoalieView.cpp: implementation of the CGoalieView class
//
///////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Goalie.h"

#include "GoalieDoc.h"
#include "GoalieView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif	// _DEBUG

extern double gFlagXPos[FLAG_COUNT];
extern double gFlagYPos[FLAG_COUNT];
extern char* gPlayMode[PLAYMODE_COUNT];

IMPLEMENT_DYNCREATE(CGoalieView, CView)

BEGIN_MESSAGE_MAP(CGoalieView, CView)
	//{{AFX_MSG_MAP(CGoalieView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_TIMER()
	ON_WM_KEYDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////////////////////////
// CGoalieView construction

CGoalieView::CGoalieView()
{
	// TODO: add construction code here
}

///////////////////////////////////////////////////////////////////////////////
// CGoalieView operations

// work version (full of garbage to be cleaned)
void CGoalieView::RedrawView()
{
	CClientDC dc(this);
	CDC dcMem;
	CBitmap bitmap, * pOldBitmap;
	CRect windowRect;
	int sizeX, sizeY;
	CPen pen, * pOldPen;
	CBrush brush, * pOldBrush;
	CPoint P[5];
	COLORREF cCol;
	double x, y;

	if (!GetDocument()->m_bGraphics) return;

	// get Device Coordinates
	dcMem.CreateCompatibleDC(&dc);

	GetClientRect(windowRect);
	sizeX = windowRect.Width();
	sizeY = windowRect.Height();
	bitmap.CreateCompatibleBitmap(&dc, sizeX, sizeY);
	bitmap.SetBitmapDimension(sizeX, sizeY);
	pOldBitmap = dcMem.SelectObject(&bitmap);
	dcMem.FillSolidRect(0, 0, sizeX, sizeY, COLOR_BACK);

	float zoom = min(float(sizeX)/SIZE_X,float(sizeY)/SIZE_Y);
	zoom = max(2,zoom);			// do not allow zoom less than 2

	#define TX(x)	int((x)*(zoom) + sizeX/2)
	#define TY(y)	int((y)*(zoom) + sizeY/2)

	brush.CreateSolidBrush(COLOR_AGENT);
	pOldBrush = dcMem.SelectObject(&brush);
	pen.CreatePen(PS_SOLID,1,RGB(0,0,0));
	pOldPen = dcMem.SelectObject(&pen);

	// lines
	dcMem.FillSolidRect(CRect(TX(gFlagXPos[LT_FLAG]), TY(gFlagYPos[LT_FLAG]),
						TX(gFlagXPos[RB_FLAG]), TY(gFlagYPos[RB_FLAG])), COLOR_FIELD);

	CPlayer* p = GetDocument()->m_goalie.GetPerception().GetAgentPlayer();
	int angle = GetDocument()->m_goalie.GetPerception().GetViewAngle();
	if (p->IsValid() && angle <= 180) {
		P[0].x = TX(p->GetAbsX());
		P[0].y = TY(p->GetAbsY());
		CVector vect(200.0, p->GetBodyDir() + p->GetHeadDir());
		vect.Rotate((double)(angle/2));
		P[1].x = TX((p->GetAbsPos() + vect).GetX());
		P[1].y = TY((p->GetAbsPos() + vect).GetY());
		vect.Rotate(-(double)angle);
		P[2].x = TX((p->GetAbsPos() + vect).GetX());
		P[2].y = TY((p->GetAbsPos() + vect).GetY());
		dcMem.Polygon(P,3);
	}

	dcMem.SelectObject(pOldPen);
	pen.DeleteObject();
	pen.CreatePen(PS_SOLID,2,COLOR_LINE);
	pOldPen= dcMem.SelectObject(&pen);

	//lines
	P[0].x = TX(gFlagXPos[LT_FLAG]); P[0].y = TY(gFlagYPos[LT_FLAG]);
	P[1].x = TX(gFlagXPos[RT_FLAG]); P[1].y = TY(gFlagYPos[RT_FLAG]);
	P[2].x = TX(gFlagXPos[RB_FLAG]); P[2].y = TY(gFlagYPos[RB_FLAG]);
	P[3].x = TX(gFlagXPos[LB_FLAG]); P[3].y = TY(gFlagYPos[LB_FLAG]);
	P[4] = P[0];
	dcMem.Polyline(P,5);

	// penalty area
	P[0].x = TX(gFlagXPos[LT_FLAG]); P[0].y = TY(gFlagYPos[LT_PEN_FLAG]);
	P[1].x = TX(gFlagXPos[LT_PEN_FLAG]); P[1].y = TY(gFlagYPos[LT_PEN_FLAG]);
	P[2].x = TX(gFlagXPos[LB_PEN_FLAG]); P[2].y = TY(gFlagYPos[LB_PEN_FLAG]);
	P[3].x = TX(gFlagXPos[LB_FLAG]); P[3].y = TY(gFlagYPos[LB_PEN_FLAG]);
	dcMem.Polyline(P,4);

	P[0].x = TX(gFlagXPos[RT_FLAG]); P[0].y = TY(gFlagYPos[RT_PEN_FLAG]);
	P[1].x = TX(gFlagXPos[RT_PEN_FLAG]); P[1].y = TY(gFlagYPos[RT_PEN_FLAG]);
	P[2].x = TX(gFlagXPos[RB_PEN_FLAG]); P[2].y = TY(gFlagYPos[RB_PEN_FLAG]);
	P[3].x = TX(gFlagXPos[RB_FLAG]); P[3].y = TY(gFlagYPos[RB_PEN_FLAG]);
	dcMem.Polyline(P,4);

	// goals area
	P[0].x = TX(gFlagXPos[LT_GOAL_FLAG]); P[0].y = TY(gFlagYPos[LT_GOAL_FLAG] - 2);
	P[1].x = TX(gFlagXPos[LT_GOAL_FLAG] + 5); P[1].y = TY(gFlagYPos[LT_GOAL_FLAG] - 2);
	P[2].x = TX(gFlagXPos[LB_GOAL_FLAG] + 5); P[2].y = TY(gFlagYPos[LB_GOAL_FLAG] + 2);
	P[3].x = TX(gFlagXPos[LB_GOAL_FLAG]); P[3].y = TY(gFlagYPos[LB_GOAL_FLAG] + 2);
	dcMem.Polyline(P,4);

	P[0].x = TX(gFlagXPos[RT_GOAL_FLAG]); P[0].y = TY(gFlagYPos[RT_GOAL_FLAG] - 2);
	P[1].x = TX(gFlagXPos[RT_GOAL_FLAG] - 5); P[1].y = TY(gFlagYPos[RT_GOAL_FLAG] - 2);
	P[2].x = TX(gFlagXPos[RB_GOAL_FLAG] - 5); P[2].y = TY(gFlagYPos[RB_GOAL_FLAG] + 2);
	P[3].x = TX(gFlagXPos[RB_GOAL_FLAG]); P[3].y = TY(gFlagYPos[RB_GOAL_FLAG] + 2);
	dcMem.Polyline(P,4);

	// goals
	dcMem.FillSolidRect(CRect(TX(gFlagXPos[LT_GOAL_FLAG]) - 2, TY(gFlagYPos[LT_GOAL_FLAG]),
					    TX(gFlagXPos[LB_GOAL_FLAG]) + 2, TY(gFlagYPos[LB_GOAL_FLAG])), COLOR_GOAL);
	dcMem.FillSolidRect(CRect(TX(gFlagXPos[RT_GOAL_FLAG]) - 2, TY(gFlagYPos[RT_GOAL_FLAG]),
					    TX(gFlagXPos[RB_GOAL_FLAG]) + 2, TY(gFlagYPos[RB_GOAL_FLAG])), COLOR_GOAL);
	dcMem.SetBkColor(COLOR_BACK);

	// rings
	dcMem.Arc(TX(gFlagXPos[L_GOAL] + 11 - 9), TY(9),
			TX(gFlagXPos[L_GOAL] + 11 + 9), TY(-9),
			TX(gFlagXPos[LT_PEN_FLAG]), TY(gFlagYPos[LC_PEN_FLAG] + 7),
			TX(gFlagXPos[LB_PEN_FLAG]), TY(gFlagYPos[LC_PEN_FLAG] - 7));
	dcMem.Arc(TX(gFlagXPos[R_GOAL] - 11 - 9), TY(9),
			TX(gFlagXPos[R_GOAL] - 11 + 9), TY(-9),
			TX(gFlagXPos[RT_PEN_FLAG]), TY(gFlagYPos[RC_PEN_FLAG] - 7),
			TX(gFlagXPos[RB_PEN_FLAG]), TY(gFlagYPos[RC_PEN_FLAG] + 7));
	dcMem.Arc(TX(-9), TY(9), TX(9), TY(-9), TX(0), TY(0), TX(0), TY(0));

	// center line
	P[0].x = TX(gFlagXPos[CT_FLAG]); P[0].y = TY(gFlagYPos[CT_FLAG]);
	P[1].x = TX(gFlagXPos[CB_FLAG]); P[1].y = TY(gFlagYPos[CB_FLAG]);
	dcMem.Polyline(P,2);

	// 11
	P[0].x = TX(gFlagXPos[L_GOAL] + 11); P[0].y = TY(0) -1;
	P[1].x = TX(gFlagXPos[L_GOAL] + 11); P[1].y = TY(0) + 1;
	dcMem.Polyline(P,2);

	P[0].x = TX(gFlagXPos[R_GOAL] - 11); P[0].y = TY(0) -1;
	P[1].x = TX(gFlagXPos[R_GOAL] - 11); P[1].y = TY(0) + 1;
	dcMem.Polyline(P,2);

	dcMem.SelectObject(pOldPen);
	pen.DeleteObject();
	pen.CreatePen(PS_SOLID,2,COLOR_FLAG);
	pOldPen = dcMem.SelectObject(&pen);

	for (int i = 0; i < FLAG_COUNT; i++) {
		dcMem.FillSolidRect(CRect(TX(gFlagXPos[i]) - 4, TY(gFlagYPos[i]) - 4,
				    TX(gFlagXPos[i]) + 4, TY(gFlagYPos[i]) + 4), COLOR_GOAL);

		P[0].x = TX(gFlagXPos[i]) - 2; P[0].y = TY(gFlagYPos[i]);
		P[1].x = TX(gFlagXPos[i]) + 2; P[1].y = TY(gFlagYPos[i]);
		dcMem.Polyline(P,2);

		P[0].x = TX(gFlagXPos[i]); P[0].y = TY(gFlagYPos[i]) - 2;
		P[1].x = TX(gFlagXPos[i]); P[1].y = TY(gFlagYPos[i]) + 2;
		dcMem.Polyline(P,2);
	}

	dcMem.SelectObject(pOldPen);
	pen.DeleteObject();
	pen.CreatePen(PS_SOLID,2,RGB(255,0,0));
	pOldPen = dcMem.SelectObject(&pen);

	CFlag** pFlag = GetDocument()->m_goalie.GetPerception().GetFlags();
	for (i = 0; i < FLAG_COUNT; i++) if (pFlag[i]->IsValid()) {
		P[3].x = TX((p->GetAbsPos() + pFlag[i]->GetRelPos()).GetX());
		P[3].y = TY((p->GetAbsPos() + pFlag[i]->GetRelPos()).GetY());
		P[0].x = P[3].x - 2; P[0].y = P[3].y;
		P[1].x = P[3].x + 2; P[1].y = P[3].y;
		dcMem.Polyline(P,2);

		P[0].x = P[3].x; P[0].y = P[3].y - 2;
		P[1].x = P[3].x; P[1].y = P[3].y + 2;
		dcMem.Polyline(P,2);
	}

	dcMem.SelectObject(pOldBrush);
	dcMem.SelectObject(pOldPen);
	brush.DeleteObject();
	pen.DeleteObject();

	brush.CreateSolidBrush(p->GetSide() == SIDE_LEFT ? COLOR_LEFT_TEAM : COLOR_RIGHT_TEAM);
	pOldBrush = dcMem.SelectObject(&brush);

	CPlayer** pPlayer = GetDocument()->m_goalie.GetPerception().GetTeammates();
	for (i = 0; i < PLAYER_COUNT / 2 - 1; i++) if (pPlayer[i]->IsValid()) {
		x = pPlayer[i]->GetAbsX();
		y = pPlayer[i]->GetAbsY();
		CVector vect(1.0, pPlayer[i]->GetBodyDir());
		dcMem.Ellipse(TX(x - 1), TY(y - 1), TX(x + 1), TY(y + 1));
		P[0].x = TX(x); P[0].y = TY(y);
		P[1].x = TX((pPlayer[i]->GetAbsPos() + vect).GetX());
		P[1].y = TY((pPlayer[i]->GetAbsPos() + vect).GetY());
		dcMem.Polyline(P,2);
	}

	dcMem.SelectObject(pOldBrush);
	brush.DeleteObject();
	brush.CreateSolidBrush(p->GetSide() == SIDE_RIGHT ? COLOR_LEFT_TEAM : COLOR_RIGHT_TEAM);
	pOldBrush = dcMem.SelectObject(&brush);

	pPlayer = GetDocument()->m_goalie.GetPerception().GetOpponents();
	for (i = 0; i < PLAYER_COUNT / 2; i++) if (pPlayer[i]->IsValid()) {
		x = pPlayer[i]->GetAbsX();
		y = pPlayer[i]->GetAbsY();
		CVector vect(1.0, pPlayer[i]->GetBodyDir());
		dcMem.Ellipse(TX(x - 1), TY(y - 1), TX(x + 1), TY(y + 1));
		P[0].x = TX(x); P[0].y = TY(y);
		P[1].x = TX((pPlayer[i]->GetAbsPos() + vect).GetX());
		P[1].y = TY((pPlayer[i]->GetAbsPos() + vect).GetY());
		dcMem.Polyline(P,2);
	}

	dcMem.SelectObject(pOldBrush);
	brush.DeleteObject();
	brush.CreateSolidBrush(COLOR_AGENT);
	pOldBrush = dcMem.SelectObject(&brush);

	pPlayer = GetDocument()->m_goalie.GetPerception().GetUnknownPlayers();
	for (i = 0; i < PLAYER_COUNT - 1; i++) if (pPlayer[i]->IsValid()) {
		x = pPlayer[i]->GetAbsX();
		y = pPlayer[i]->GetAbsY();
		CVector vect(1.0, pPlayer[i]->GetBodyDir());
		dcMem.Ellipse(TX(x - 1), TY(y - 1), TX(x + 1), TY(y + 1));
		P[0].x = TX(x); P[0].y = TY(y);
		P[1].x = TX((pPlayer[i]->GetAbsPos() + vect).GetX());
		P[1].y = TY((pPlayer[i]->GetAbsPos() + vect).GetY());
		dcMem.Polyline(P,2);
	}

	dcMem.SelectObject(pOldBrush);
	brush.DeleteObject();

	// draw a ball
	CBall* pBall = GetDocument()->m_goalie.GetPerception().GetBall();
	if (pBall->IsValid()) {
		if (pBall->GetAge() < 2) cCol = COLOR_BALL;
		else cCol = COLOR_ANY_OBJECT;
		brush.CreateSolidBrush(cCol);
		pOldBrush = dcMem.SelectObject(&brush);
		x = pBall->GetAbsX();
		y = pBall->GetAbsY();
		dcMem.Ellipse(TX(x - 0.7), TY(y - 0.7), TX(x + 0.7), TY(y + 0.7));
		dcMem.SelectObject(pOldBrush);
		brush.DeleteObject();

	// future ball positions
		brush.CreateSolidBrush(COLOR_FIELD);
		pOldBrush = dcMem.SelectObject(&brush);

		for(i = 1; i < MAX_FUTURE_POSITIONS; i++) {
			CPosition futPos = pBall->GetFuturePos(i);
			x = futPos.GetX();
			y = futPos.GetY();
			dcMem.Ellipse(TX(x - 0.3), TY(y - 0.3), TX(x + 0.3), TY(y + 0.3));
		}

		dcMem.SelectObject(pOldBrush);
		brush.DeleteObject();

	// ball intercept position
		brush.CreateSolidBrush(COLOR_LINE);
		pOldBrush = dcMem.SelectObject(&brush);
		CPosition intPos = GetDocument()->m_goalie.GetPerception().GetBallInterceptPos(p);
		x = intPos.GetX();
		y = intPos.GetY();
		dcMem.Ellipse(TX(x - 0.5), TY(y - 0.5), TX(x + 0.5), TY(y + 0.5));
		dcMem.SelectObject(pOldBrush);
		brush.DeleteObject();
	}

/*	// branky

	Brush.CreateSolidBrush(cCol);
	oldBrush = dcMem.SelectObject(&Brush);
	x = per->m_pGoal[0]->GetAbsX();
	y = per->m_pGoal[0]->GetAbsY();	
	dcMem.Ellipse(TX(x-1),TY(y-1),TX(x+1),TY(y+1));	
	dcMem.SelectObject(oldBrush);
	Brush.DeleteObject();

	Brush.CreateSolidBrush(cCol);
	oldBrush = dcMem.SelectObject(&Brush);
	x = per->m_pGoal[1]->GetAbsX();
	y = per->m_pGoal[1]->GetAbsY();	
	dcMem.Ellipse(TX(x-1),TY(y-1),TX(x+1),TY(y+1));	
	dcMem.SelectObject(oldBrush);
	Brush.DeleteObject();
*/

	// copy the memory bitmap into the window
	dc.BitBlt(0, 0, sizeX, sizeY, &dcMem, 0, 0, SRCCOPY);
	// remove the created bitmap
	dcMem.SelectObject(pOldBitmap);
	dcMem.DeleteDC();
	bitmap.DeleteObject();
}

///////////////////////////////////////////////////////////////////////////////
// CGoalieView overrides

void CGoalieView::OnDraw(CDC* pDC)
{
	CGoalieDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
	pDC->SetBkColor(COLOR_BACK);
	RedrawView();
}

BOOL CGoalieView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

///////////////////////////////////////////////////////////////////////////////
// CGoalieView message handlers

int CGoalieView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	m_uTimerID = SetTimer(1, 100, NULL);

	return 0;
}

void CGoalieView::OnDestroy()
{
	KillTimer(m_uTimerID);
	CView::OnDestroy();
	// TODO: Add your message handler code here
}

void CGoalieView::OnTimer(UINT nIDEvent)
{
	// TODO: Add your message handler code here and/or call default
//	GetDocument()->m_goalie.Play();
	Invalidate(FALSE);
	CView::OnTimer(nIDEvent);
}

///////////////////////////////////////////////////////////////////////////////
// CGoalieView implementation

CGoalieView::~CGoalieView()
{
}

///////////////////////////////////////////////////////////////////////////////
// CGoalieView diagnostics

#ifdef _DEBUG

void CGoalieView::AssertValid() const
{
	CView::AssertValid();
}

void CGoalieView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CGoalieDoc* CGoalieView::GetDocument()	// non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGoalieDoc)));
	return (CGoalieDoc*)m_pDocument;
}

#endif	//_DEBUG

void CGoalieView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
  if (nChar >= '0' && nChar <= '9')
    GetDocument()->m_goalie.SetPlayer(nChar - '0');
	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}
