// ParseString.h: interface for the CParseString class
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __AFX_PARSESTRING_H_
#define __AFX_PARSESTRING_H_

#if _MSC_VER > 1000
#pragma once
#endif	// _MSC_VER > 1000

// ParseString.h header file declares following classes

class CParseString;

#define STR_OK				0	// operation successful
#define STR_ERR_EMPTY		1	// string empty
#define STR_ERR_LBRACKET	2	// missing left opening bracket
#define STR_ERR_RBRACKET	3	// missing right closing bracket
#define STR_ERR_INT			4	// missing correct integer value
#define STR_ERR_DOUBLE		5	// missing correct floating point value

///////////////////////////////////////////////////////////////////////////////
// Class: CParseString
// Date: 12.04.01
// Version Date: 21.04.01
// Version: 1.04
// Author: Marek Lehuta
//
// Base Class: CString
///////////////////////////////////////////////////////////////////////////////
// Description: adds required parsing functionality to CString
///////////////////////////////////////////////////////////////////////////////

class CParseString : public CString
{
// Construction
public:
	CParseString() { m_iPos = 0; };
	CParseString(LPCTSTR pStr) : CString(pStr) { m_iPos = 0; };
	CParseString(const CString& str);

// Attributes
private:
	int m_iErrorCode;
	int m_iPos;

// Operations
public:
	int GetLastError() const { return m_iErrorCode; };

	void IncPosition(int iPos = 1);
	void DecPosition(int iPos = 1);
	void SetPosition(int iPos);
	int GetPosition() const { return m_iPos; };

	CString ParseBracket();
	CString ParseString();
	char ParseChar();
	int ParseInteger();
	double ParseDouble();

	const CParseString& operator=(const CString& str);

private:
	void OverreadLeadingSpaces();

// Implementation
public:
	virtual ~CParseString() {};

};

///////////////////////////////////////////////////////////////////////////////

#endif	// __AFX_PARSESTRING_H_
