// Perception.cpp: implementation of the CPerception class
//
///////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Goalie.h"
#include "Perception.h"
#include "PlayerAgent.h"
#include "ComplexAgent.h"

#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#endif	// _DEBUG

extern double gFlagXPos[FLAG_COUNT];
extern double gFlagYPos[FLAG_COUNT];
extern char* gPlayMode[PLAYMODE_COUNT];

///////////////////////////////////////////////////////////////////////////////
// CPerception construction

CPerception::CPerception()
{
	int	i;

	// create the needed objects
	// players
	for (i = 0; i < PLAYER_COUNT; i++) m_pPlayer[i] = new CPlayer;
	for (i = 0; i < PLAYER_COUNT - 1; i++) {
		m_pUnknownPlayer[i] = new CPlayer;
		m_pTmpPlayer[i] = new CPlayer;
	}

	// ball
	m_pBall = new CBall;

	// goals
	m_pGoal[0] = new CGoal(GOAL_LEFT);
	m_pGoal[1] = new CGoal(GOAL_RIGHT);

	// flags
	for (i = 0; i < FLAG_COUNT; i++) m_pFlag[i] = new CFlag;

	// lines
	for (i = 0; i < 4; i++) m_pLine[i] = new CLine;

	m_iSimTime = m_iSeeTime = m_iHearTime = m_iSenseTime = 0;
	m_iPlayMode = PM_UNKNOWN;
	m_iPlayerCount = 0;
	m_iFlagCount = 0;

	m_iViewQuality = 1;
	m_iViewAngle = 90;

	m_dEffort = 1.0;
	m_dSpeed = 0.0;

	m_bDebug = false;
}

///////////////////////////////////////////////////////////////////////////////
// CPerception operations

CPlayer** CPerception::GetUnknownPlayers() const
{
	return (CPlayer**)m_pUnknownPlayer;
}

CGoal* CPerception::GetGoal(bool bOur)
{
	int	index = 0;

	// which one is the correct goal
	if ((m_pPlayer[0]->GetSide() == SIDE_LEFT && !bOur) ||
		(m_pPlayer[0]->GetSide() == SIDE_RIGHT && bOur)) index = 1;
	if (!m_pGoal[index]->IsValid()) m_pGoal[index]->SetID(index + 1);

	return m_pGoal[index];
}

double CPerception::GetBallDistance(CPlayer* pPlayer) const
{
	if (pPlayer == NULL) pPlayer = m_pPlayer[0];
	return pPlayer->GetDistanceTo(m_pBall);
}

double CPerception::GetBallDirection(CPlayer* pPlayer) const
{
	if (pPlayer == NULL) pPlayer = m_pPlayer[0];
	return pPlayer->GetAngleTo(m_pBall);
}

double CPerception::GetGoalDistance(CPlayer* pPlayer, bool bOwn)
{
	int	index = 0;

	if (pPlayer == NULL) pPlayer = m_pPlayer[0];
	// which one is the correct goal
	if ((pPlayer->GetSide() == SIDE_LEFT && !bOwn) ||
		(pPlayer->GetSide() == SIDE_RIGHT && bOwn)) index = 1;

	// if the player can see the goal, then return visible distance
	if (m_pGoal[index]->IsValid()) return m_pGoal[index]->GetDistance();

	m_pGoal[index]->SetID(index + 1);
	// the goal is not visible, so use the absolute position of the agent
	return pPlayer->GetDistanceTo(m_pGoal[index]);
}

double CPerception::GetGoalDirection(CPlayer* pPlayer, bool bOwn)
{
	int	index = 0;

	if (pPlayer == NULL) pPlayer = m_pPlayer[0];
	// which one is the correct goal
	if ((m_pPlayer[0]->GetSide() == SIDE_LEFT && !bOwn) ||
		(m_pPlayer[0]->GetSide() == SIDE_RIGHT && bOwn)) index = 1;

	// if the player can see the goal, then return visible direction
	if (m_pGoal[index]->IsValid()) return (m_pGoal[index]->GetRelPos() - m_pPlayer[0]->GetBodyDir()).GetAngle();

	m_pGoal[index]->SetID(index + 1);
	// the goal is not visible, so use the absolute position of the agent
	return pPlayer->GetAngleTo(m_pGoal[index]);
}

bool CPerception::IsInPenaltyArea(CMovableObject* pObject, bool bOur) const
{
	bool bLeft = false;

	if (pObject == NULL) pObject = m_pPlayer[0];
	if (bOur && m_pPlayer[0]->GetSide() == SIDE_LEFT) bLeft = true;
	if (!bOur && m_pPlayer[0]->GetSide() == SIDE_RIGHT) bLeft = true;

	if (bLeft) {
		if ((pObject->GetAbsX() <= -PW) && (pObject->GetAbsX() >= -FW) &&
			(pObject->GetAbsY() <= PH) && (pObject->GetAbsY() >= -PH)) return true;
		else return false;
	}
	else {
		if ((pObject->GetAbsX() >= PW) && (pObject->GetAbsX() <= FW) &&
			(pObject->GetAbsY() <= PH) && (pObject->GetAbsY() >= -PH)) return true;
		else return false;
	}
}

bool CPerception::IsOnOurHalf(CMovableObject* pObject) const
{
	if (pObject == NULL) pObject = m_pPlayer[0];
	if (m_pPlayer[0]->GetSide() == SIDE_LEFT) {
		if (pObject->GetAbsX() <= 0.0) return true;
		else return false;
	}
	else {
		if (pObject->GetAbsX() >= 0.0) return true;
		else return false;
	}
}

CPlayer* CPerception::GetBallNearestPlayer() const
{
	CPlayer* pNearestPlayer = NULL;
	int iMinTime = 1000;
	int iPlayerTime = 0;

	// searching among known players
	for (int i = 0; i < PLAYER_COUNT; i++) if (m_pPlayer[i]->IsValid()) {
		iPlayerTime = GetBallInterceptTime(m_pPlayer[i]);
		if (iPlayerTime < iMinTime) {
			iMinTime = iPlayerTime;
			pNearestPlayer = m_pPlayer[i];
		}
	}

	// searching among unknown players
	for (i = 0; i < PLAYER_COUNT - 1; i++) if (m_pUnknownPlayer[i]->IsValid()) {
		iPlayerTime = GetBallInterceptTime(m_pUnknownPlayer[i]);
		if (iPlayerTime < iMinTime) {
			iMinTime = iPlayerTime;
			pNearestPlayer = m_pUnknownPlayer[i];
		}
	}

	return pNearestPlayer;
}

CPlayer* CPerception::GetBallNearestTeammate() const
{
	CPlayer* pNearestTeammate = NULL;
	int iMinTime = 1000;
	int iPlayerTime = 0;

	// searching among our players
	for (int i = 0; i < PLAYER_COUNT / 2; i++) if (m_pPlayer[i]->IsValid()) {
		iPlayerTime = GetBallInterceptTime(m_pPlayer[i]);
		if (iPlayerTime < iMinTime) {
			iMinTime = iPlayerTime;
			pNearestTeammate = m_pPlayer[i];
		}
	}

	return pNearestTeammate;
}

CPlayer* CPerception::GetBallNearestOpponent() const
{
	CPlayer* pNearestOpponent = NULL;
	int iMinTime = 1000;
	int iPlayerTime = 0;

	// searching among our players
	for (int i = PLAYER_COUNT / 2; i < PLAYER_COUNT; i++) if (m_pPlayer[i]->IsValid()) {
		iPlayerTime = GetBallInterceptTime(m_pPlayer[i]);
		if (iPlayerTime < iMinTime) {
			iMinTime = iPlayerTime;
			pNearestOpponent = m_pPlayer[i];
		}
	}

	return pNearestOpponent;
}

int CPerception::GetBallInterceptTime(CPlayer* pPlayer) const
{
	int iTime = 1;
	double dPower = 100.0;
	double dActionRadius;
	double dBallDistance;
	CPosition futureBallPos;
	double dAngle = 0;
	bool bReady = false;
	bool bFirst = false;

	if (pPlayer == NULL) pPlayer = m_pPlayer[0];
	while (!bReady && (iTime < MAX_FUTURE_POSITIONS)) {
		futureBallPos = m_pBall->GetFuturePos(iTime);
		dBallDistance = pPlayer->GetDistanceTo(futureBallPos);

		// nemozeme utekat do miesta, smerom ku ktoremu nevidiet loptu
		dAngle = pPlayer->GetAngleTo(futureBallPos);
		if (((dAngle > 40) || (dAngle < -40)) && bFirst == false) bFirst = true;
		else {
			dActionRadius = pPlayer->GetFutureDistanceActive(iTime, dPower,
				pPlayer->GetSpeedDist(), GetEffort());
			if (dActionRadius + KICKABLE_MARGIN > dBallDistance) bReady = true;
		}
		iTime++;
	}

	return iTime - 1;
}

CPosition CPerception::GetBallInterceptPos(CPlayer* pPlayer) const
{
	int iTime = 1;
	double dPower = 100.0;
	double dActionRadius;
	double dBallDistance;
	CPosition futureBallPos;
	double dAngle = 0;
	bool bReady = false;
	bool bFirst = false;

	if (pPlayer == NULL) pPlayer = m_pPlayer[0];
	while (!bReady && iTime < MAX_FUTURE_POSITIONS) {
		futureBallPos = m_pBall->GetFuturePos(iTime);
		dBallDistance = pPlayer->GetDistanceTo(futureBallPos);

		// nemozeme utekat do miesta, smerom ku ktoreme nevidiet loptu
		dAngle = pPlayer->GetAngleTo(futureBallPos);
    bFirst = (fabs(dAngle) > TURN_TOL);
  	dActionRadius = pPlayer->GetFutureDistanceActive(iTime, dPower, bFirst);
		if (dActionRadius + KICKABLE_MARGIN > dBallDistance) bReady = true;
		iTime++;
	}
/*
	return futureBallPos;

	return m_pBall->GetFuturePos(1);

	// toto mi zhorsilo spravanie hraca ...

	CPosition futureBallPos;
	CPosition minFutureBallPos;
	double dBallDistance;
	double dDelay;
	double dMinDelay = 999999.9;

	for (int step = 1; step < MAX_FUTURE_POSITIONS; step++) {
		futureBallPos = m_pBall->GetFuturePos(step);
		dBallDistance = (pPlayer->GetAbsPos() - futureBallPos).GetLength();
		if (dBallDistance > step * 0.2) dDelay = dBallDistance / 0.2 + 1;
		else dDelay = step;
		if (dDelay < dMinDelay) {
			minFutureBallPos = futureBallPos;
			dMinDelay = dDelay;
		}
	}
*/
	return futureBallPos;
}

bool CPerception::IsOurPlayer(CPlayer* pPlayer) const
{
	return pPlayer->GetSide() == m_pPlayer[0]->GetSide();
}

CPlayer* CPerception::GetNearestTeammateTo(const CPosition& pos) const
{
	CPlayer* pNearestTeammate = NULL;
	double dDist;
	double dMinDist = 1000;

	for (int i = 0; i < PLAYER_COUNT / 2; i++) if (m_pPlayer[i]->IsValid()) {
		dDist = m_pPlayer[i]->GetDistanceTo(pos);
		if (dDist < dMinDist) {
			pNearestTeammate = m_pPlayer[i];
			dMinDist = dDist;
		}
	}

	return pNearestTeammate;
}

CPlayer* CPerception::GetNearestOpponentTo(const CPosition& pos) const
{
	CPlayer* pNearestOpponent = NULL;
	double dDist;
	double dMinDist = 1000;

	for (int i = PLAYER_COUNT / 2; i < PLAYER_COUNT; i++) if (m_pPlayer[i]->IsValid()) {
		dDist = m_pPlayer[i]->GetDistanceTo(pos);
		if (dDist < dMinDist) {
			pNearestOpponent = m_pPlayer[i];
			dMinDist = dDist;
		}
	}

	return pNearestOpponent;
}

CPlayer* CPerception::GetNearestTeammate(CPlayer* pPlayer) const
{
	CPlayer* pNearestTeammate = NULL;
	double dDist;
	double dMinDist = 1000;

	if (pPlayer == NULL) pPlayer = m_pPlayer[0];
	for (int i = 0; i < PLAYER_COUNT / 2; i++) if (m_pPlayer[i]->IsValid() && (m_pPlayer[i] != pPlayer)) {
		dDist = pPlayer->GetDistanceTo(m_pPlayer[i]);
		if (dDist < dMinDist) {
			pNearestTeammate = m_pPlayer[i];
			dMinDist = dDist;
		}
	}

	return pNearestTeammate;
}

CPlayer* CPerception::GetNearestOpponent(CPlayer* pPlayer) const
{
	CPlayer* pNearestOpponent = NULL;
	double dDist;
	double dMinDist = 1000;

	if (pPlayer == NULL) pPlayer = m_pPlayer[0];
	for (int i = PLAYER_COUNT / 2; i < PLAYER_COUNT; i++) if (m_pPlayer[i]->IsValid() && (m_pPlayer[i] != pPlayer)) {
		dDist = pPlayer->GetDistanceTo(m_pPlayer[i]);
		if (dDist < dMinDist) {
			pNearestOpponent = m_pPlayer[i];
			dMinDist = dDist;
		}
	}

	return pNearestOpponent;
}

CPlayer* CPerception::GetBetterPlacedTeammate()
{
	CPlayer* pThisPlayer = GetAgentPlayer();
	CPlayer* pThisPlayerNearestOpponent = GetNearestOpponent(pThisPlayer);

	if (pThisPlayerNearestOpponent == NULL) return NULL;

	double dThisPlayerDist = pThisPlayer->GetDistanceTo(pThisPlayerNearestOpponent);

	CGoal* pGoal = GetGoal();
	double dBestQuality = -100;
	CPlayer* pBestPlayer = NULL;
	CPlayer* pPlayer = NULL;

	for (int i = 0; i < PLAYER_COUNT / 2; i++) if (m_pPlayer[i]->IsValid()) {
		double dQuality = 0;
		pPlayer = m_pPlayer[i];
    if (pThisPlayer->GetDistanceTo(pPlayer) > MAX_TEAMMATE_DIST) continue;
    double coef = (pThisPlayer->GetAbsX() * m_pPlayerAgent->GetSwitch() < 0) ? 16 : 8;
    double diff = pPlayer->GetDistanceTo(pGoal) - pThisPlayer->GetDistanceTo(pGoal);
    if (pThisPlayer->GetAbsX() * m_pPlayerAgent->GetSwitch() < -20 && diff > 0) continue;
    dQuality += diff * coef;
		dQuality += (pPlayer->GetDistanceTo(GetNearestOpponent(pPlayer)) - dThisPlayerDist) * 8;
		dQuality -= (MAX_TEAMMATE_DIST - pThisPlayer->GetDistanceTo(pPlayer)) * 2;
    if (IsInPenaltyArea(pThisPlayer, FALSE)) dQuality -= 50;
    if (dQuality > dBestQuality) {
			dBestQuality = dQuality;
			pBestPlayer = pPlayer;
		}
	}

	if (dBestQuality > 0) return pBestPlayer;
	else return NULL;
}

double CPerception::GetAvoidAngle(const CPosition& destPos, CPlayer* pPlayer) const
{
	double dMinAng, dMaxAng, dAng, dAng1, dAng2, dDelAng;
	double dDist;
	CPosition pos;

	if (pPlayer == NULL) pPlayer = m_pPlayer[0];
	CPosition pos0 = pPlayer->GetAbsPos();
	double dAng0 = pPlayer->GetAngleTo(destPos);
	double dDist0 = pPlayer->GetDistanceTo(destPos);

	dMinAng = dMaxAng = 0;
	for (int i = 0; i < PLAYER_COUNT; i++) if (m_pPlayer[i]->IsValid() && (m_pPlayer[i] != pPlayer)) {
		pos = m_pPlayer[i]->GetAbsPos();
		dDist = pPlayer->GetDistanceTo(pos);
		if (dDist < dDist0) {
			dAng = pPlayer->GetAngleTo(pos) - dAng0;
			if (dDist < 2) dDelAng = 30;	// 1.2) dDelAng = 30;
			else if (dDist < 10) dDelAng = 15;
			else if (dDist < 20) dDelAng = 5;
			else dDelAng = 2;
			dAng1 = dAng - dDelAng;
			dAng2 = dAng + dDelAng;
			if (((dAng1 > 0) && (dAng2 > 0)) || ((dAng1 < 0) && (dAng2 < 0))) dAng1 = dAng2 = 0;

			if (dMaxAng < dAng2) dMaxAng = dAng2;
			if (dMinAng > dAng1) dMinAng = dAng1;
		}
	}

	return (-dMinAng < dMaxAng ? dMinAng + dAng0 : dMaxAng + dAng0);
}

bool CPerception::ParseMessage(LPCTSTR psMsg)
{
	CParseString msg(psMsg), msgType;

	msg = msg.ParseBracket();
	msgType = msg.ParseString();

	// message at init and reconnect
	if (msgType == "init") return ParseInit(msg);
	// something to see
	if (msgType == "see") return ParseSee(msg);
	// something to hear
	if (msgType == "hear") return ParseHear(msg);
	// sense body
	if (msgType == "sense_body") return ParseSenseBody(msg);

	return true;
}

void CPerception::UpdateAgent(LPCTSTR psMsg)
{
	CParseString msg(psMsg), msgType;
	double dArg1, dArg2;

	msg = msg.ParseBracket();
	msgType = msg.ParseString();
	dArg1 = msg.ParseDouble();
	dArg2 = msg.ParseDouble();

	// move message sent
	if (msgType == "move") {
		if (m_pPlayer[0]->GetSide() == SIDE_RIGHT) {
			dArg1 = -dArg1;
			dArg2 = -dArg2;
		}
		m_pPlayer[0]->SetAbsPos(dArg1, dArg2);
		m_pPlayer[0]->SetSpeedDist(0.0);
		if ((m_iPlayMode == PM_FREE_KICK_L) && (m_pPlayer[0]->GetSide() == SIDE_LEFT) ||
			(m_iPlayMode == PM_FREE_KICK_R) && (m_pPlayer[0]->GetSide() == SIDE_RIGHT))
		{
			m_pBall->SetAbsPos(dArg1, dArg2);
			m_pBall->SetSpeedDist(0.0);
		}
	}
	// dash message sent
	else if (msgType == "dash") {
		m_pPlayer[0]->SetSpeedDist(m_pPlayer[0]->GetSpeedDist() + dArg1 * DASH_POWER_RATE);
	}
	// kick message sent
	else if (msgType == "kick") {
		if (m_pBall->GetDistance() < KICKABLE_MARGIN + PLAYER_SIZE + BALL_SIZE) {
			dArg1 *= 1 - 0.25 * (m_pPlayer[0]->GetAngleTo(m_pBall)) / 180 -
				0.25 * (m_pBall->GetDistance() - PLAYER_SIZE - BALL_SIZE) / KICKABLE_MARGIN;
			m_pBall->SetSpeedVector(m_pBall->GetSpeedDist() + dArg1 * KICK_POWER_RATE, m_pPlayer[0]->GetBodyDir() + dArg2);
		}
	}
	// turn message sent
	else if (msgType == "turn") {
		if (dArg1 > 180) dArg1 = 180;
		else if (dArg1 < -180) dArg1 = -180;
		dArg2 = dArg1 / (1 + INERTIA_MOMENT * m_pPlayer[0]->GetSpeedDist());
		m_pPlayer[0]->SetBodyDir(m_pPlayer[0]->GetBodyDir() + dArg2);
		m_pPlayer[0]->SetSpeedDir(m_pPlayer[0]->GetSpeedDir() + dArg2);
	}
	// turn neck message sent
	else if (msgType == "turn_neck") {
		m_pPlayer[0]->SetHeadDir(m_pPlayer[0]->GetHeadDir() + dArg1);
	}
	// catch message sent
	else if (msgType == "catch") {
	}
	// change view message sent
	else if (msgType == "change_view") {
		msgType = msg.ParseString();
		if (msgType == "narrow") m_iViewAngle = 45;
		else if (msgType == "wide") m_iViewAngle = 180;
		else m_iViewAngle = 90;

		msgType = msg.ParseString();
		if (msgType == "low") m_iViewQuality = 0;
		else m_iViewQuality = 1;
	}
}

void CPerception::UpdateField()
{
	CPosition refPos;
	CVector agentSpeed = m_pPlayer[0]->GetSpeed();
	int i;

	for (i = 0; i < m_iFlagCount; i++) m_pFlag[i]->SetRelPos(m_pFlag[i]->GetRelPos() - agentSpeed);

	m_pPlayer[0]->UpdatePos(CPosition());
	refPos = m_pPlayer[0]->GetAbsPos();
	for (i = 1; i < PLAYER_COUNT; i++) {
		if (m_pPlayer[i]->IsValid()) m_pPlayer[i]->UpdatePos(refPos);
	}
	for (i = 0; i < PLAYER_COUNT - 1; i++) {
		if (m_pUnknownPlayer[i]->IsValid()) m_pUnknownPlayer[i]->UpdatePos(refPos);
	}
	if (m_pBall->IsValid()) m_pBall->UpdatePos(refPos);
}

void CPerception::UnlockAll()
{
	int i;
	double dBodyDir = m_pPlayer[0]->GetBodyDir();

	for (i = 1; i < PLAYER_COUNT; i++) if (m_pPlayer[i]->IsValid()) {
		if (!m_pPlayer[i]->IsLocked() && fabs(m_pPlayer[0]->GetHeadAngleTo(m_pPlayer[i])) <= (double)m_iViewAngle / 2)
			m_pPlayer[i]->Invalidate();
		else if (m_pPlayer[i]->IsLocked()) m_pPlayer[i]->Unlock();
	}
	for (i = 0; i < PLAYER_COUNT - 1; i++) if (m_pUnknownPlayer[i]->IsValid()) {
		if (!m_pUnknownPlayer[i]->IsLocked() && fabs(m_pPlayer[0]->GetHeadAngleTo(m_pUnknownPlayer[i])) <= (double)m_iViewAngle / 2)
			m_pUnknownPlayer[i]->Invalidate();
		else if (m_pUnknownPlayer[i]->IsLocked()) m_pUnknownPlayer[i]->Unlock();
	}
	if (m_pBall->IsValid()) {
		if (!m_pBall->IsLocked() &&
			fabs(m_pPlayer[0]->GetHeadAngleTo(m_pBall->GetFuturePos(m_iSeeTime - m_pBall->GetSimTime()))) <= (double)m_iViewAngle / 2)
		{
//		if (!m_pBall->IsLocked() && fabs(m_pBall->GetDirection() - dBodyDir) <= m_iViewAngle / 2) {
			m_pBall->Invalidate();	// critical situation, no idea of ball's whereabouts
		}
		else if (m_pBall->IsLocked()) m_pBall->Unlock();
	}
}

bool CPerception::ParseInit(CParseString& msg)
{
	CParseString str;
	int mode = 1;

	// get the side
	if (msg.ParseChar() == 'l') m_pPlayer[0]->SetSide(SIDE_LEFT);
	else m_pPlayer[0]->SetSide(SIDE_RIGHT);

	// get the number of the player
	m_pPlayer[0]->SetNumber(msg.ParseInteger());
	if (msg.GetLastError() != STR_OK) return false;

	// get the playmode
	str = msg.ParseString();
	if (msg.GetLastError() != STR_OK) return false;

	while ((mode < PLAYMODE_COUNT) && (str.Find(gPlayMode[mode]) == -1)) mode++;
	if (mode < PLAYMODE_COUNT) m_iPlayMode = mode;
	else m_iPlayMode = PM_UNKNOWN;

	return true;
}

bool CPerception::ParseSee(CParseString& msg)
{
	// delete previously seen players
	InvalidateAll();

	//kvoli hladaniu lopty, aby sme vedeli, ze bol vnem
	SetNewPerception(true);

	// get the time
	m_iSimTime = m_iSeeTime = msg.ParseInteger();
	if (msg.GetLastError() != STR_OK) return false;
	m_pPlayer[0]->SetTime(m_iSeeTime);

	// get the visible objects
	while (msg.GetPosition() < msg.GetLength()) {
		if (!ParseObject((CParseString)msg.ParseBracket())) {
#ifdef _DEBUG
//			CString error;
//			error.Format(" : error parsing object info at position %d !\n", strObject.GetPosition());
//			afxDump << error << strObject;
#endif
		}
	}

	// get the absolute localization of the player
	LocalizeAgent();

	// calculate the absolute positions of the other moveable objects perceived
	LocalizeAll();
	UnlockAll();

	// true, although maybe not all objects could be parsed
	return true;
}

bool CPerception::ParseHear(CParseString& msg)
{
	CParseString str;
	int mode = 1;

	// get the time
	m_iSimTime = m_iHearTime = msg.ParseInteger();
	if (msg.GetLastError() != STR_OK) return false;

	// get the source
	str = msg.ParseString();
	if (str == "referee") {
		// it is from the referee
		// get the message
		str = msg.ParseString();
		if (msg.GetLastError() != STR_OK) return false;

		while ((mode < PLAYMODE_COUNT) && (str.Find(gPlayMode[mode]) == -1)) mode++;
		if (mode < PLAYMODE_COUNT) m_iPlayMode = mode;
		else m_iPlayMode = PM_UNKNOWN;
	}
	else if (str == "self") {
		// it has been sent by myself
	}
	else {
		// it is a say message
	}

	return true;
}

bool CPerception::ParseSenseBody(CParseString& msg)
{
	CParseString str;
	CString tmp;

	// read the time
	m_iSimTime = m_iSenseTime = msg.ParseInteger();
	if (msg.GetLastError() != STR_OK) return false;

	// read the view mode
	str = msg.ParseBracket();
	str.ParseString();
	tmp = str.ParseString();
	if (str.GetLastError() != STR_OK) return false;
	if (tmp == "low") m_iViewQuality = 0;
	else m_iViewQuality = 1;

	tmp = str.ParseString();
	if (str.GetLastError() != STR_OK) return false;
	if (tmp == "narrow") m_iViewAngle = 45;
	else if (tmp == "wide") m_iViewAngle = 180;
	else m_iViewAngle = 90;

	// read the stamina
	str = msg.ParseBracket();
	str.ParseString();
	m_iStamina = str.ParseInteger();
	if (str.GetLastError() != STR_OK) return false;

	// read the effort
	m_dEffort = str.ParseDouble();
	if (str.GetLastError() != STR_OK) return false;
	if (m_dEffort > 1.0) m_dEffort = 1.0;
	if (m_dEffort < EFFORT_MIN) m_dEffort = EFFORT_MIN;

	// read the speed
	str = msg.ParseBracket();
	str.ParseString();
	m_dSpeed = str.ParseDouble();
	if (str.GetLastError() != STR_OK) return false;
	if (m_dSpeed > PLAYER_SPEED_MAX * PLAYER_DECAY) m_dSpeed = PLAYER_SPEED_MAX * PLAYER_DECAY;
	m_pPlayer[0]->SetSpeedDist(m_dSpeed);

	// read the angle of head
	str = msg.ParseBracket();
	str.ParseString();
	m_pPlayer[0]->SetHeadDir(m_dHeadAng = str.ParseDouble());
	if (str.GetLastError() != STR_OK) return false;

	// read the counts
	str = msg.ParseBracket();
	str.ParseString();
	m_iKickCount = str.ParseInteger();
	if (str.GetLastError() != STR_OK) return false;

	str = msg.ParseBracket();
	str.ParseString();
	m_iDashCount = str.ParseInteger();
	if (str.GetLastError() != STR_OK) return false;

	str = msg.ParseBracket();
	str.ParseString();
	m_iTurnCount = str.ParseInteger();
	if (str.GetLastError() != STR_OK) return false;

	str = msg.ParseBracket();
	str.ParseString();
	m_iSayCount = str.ParseInteger();
	if (str.GetLastError() != STR_OK) return false;
	
	str = msg.ParseBracket();
	str.ParseString();
	m_iTurnNeckCount = str.ParseInteger();
	if (str.GetLastError() != STR_OK) return false;

	return true;
}

bool CPerception::ParseObject(CParseString& msg)
{
	CParseString str;
	CString tmp;
	int iMsgPos;

	iMsgPos = msg.GetPosition();
	str = msg.ParseBracket();
	tmp = str.ParseString();
	msg.SetPosition(iMsgPos);

	if (tmp == "player" || tmp == "Player") {
		m_pTmpPlayer[m_iPlayerCount]->ParseInfo(msg);
		tmp = str.ParseString();
		if (str.GetLastError() == STR_ERR_EMPTY) m_pTmpPlayer[m_iPlayerCount]->SetSide(SIDE_UNKNOWN);
		else if (tmp == m_strTeam) m_pTmpPlayer[m_iPlayerCount]->SetSide(m_pPlayer[0]->GetSide());
		else {
			if (m_pPlayer[0]->GetSide() == SIDE_LEFT) m_pTmpPlayer[m_iPlayerCount]->SetSide(SIDE_RIGHT);
			else m_pTmpPlayer[m_iPlayerCount]->SetSide(SIDE_LEFT);
		}
		m_pTmpPlayer[m_iPlayerCount++]->SetTime(m_iSeeTime);
	}
	else if (tmp == "ball" || tmp == "Ball") {
		m_pBall->ParseInfo(msg);
		m_pBall->SetTime(m_iSeeTime);
	}
	else if (tmp == "goal" || tmp == "oal") {
		if (str.ParseChar() == 'l') m_pGoal[0]->ParseInfo(msg);
		else m_pGoal[1]->ParseInfo(msg);
	}
	else if (tmp == "flag" || tmp == "lag") m_pFlag[m_iFlagCount++]->ParseInfo(msg);
	else if (tmp == "line" || tmp == "ine") {
		switch (str.ParseChar()) {
			case 'l':	m_pLine[0]->ParseInfo(msg);
			case 'r':	m_pLine[1]->ParseInfo(msg);
			case 't':	m_pLine[2]->ParseInfo(msg);
			case 'b':	m_pLine[3]->ParseInfo(msg);
		}
	}

	return true;
}

void CPerception::InvalidateAll()
{
	int	i;

	// invalidate temporary players
	m_iPlayerCount = 0;
	for (i = 0; i < PLAYER_COUNT - 1; i++) m_pTmpPlayer[i]->Invalidate();

	m_pGoal[0]->Invalidate();
	m_pGoal[1]->Invalidate();

	// and other objects
	m_iFlagCount = 0;
	for (i = 0; i < FLAG_COUNT; i++) m_pFlag[i]->Invalidate();

	for (i = 0; i < 4; i++) m_pLine[i]->Invalidate();
}

bool CPerception::LocalizeAgent()
{
	int i;
	int iFlag1;
	int iFlag2;
	double iMinAngle = 180;		//
	double iMaxAngle = -180;	//
	double tmpAng;				//
	double r1, r2;				// the distance of the player to the flags 
	double dist, dist2;			// the distance (and square) of the two flags
	double a;					// distance from one flag to intersection point P3
	double h;					// distance from P3 to the intersection Points P1 and P2 of the two circles
//	double dMaxDist = 0;		// distance of the most distant line
	CPosition p1, p2, p3;		//
	CVector vect;				//
/*
	if (m_pLine[0]->IsValid() && (dMaxDist < m_pLine[0]->GetDistance())) {	// left line
		dMaxDist = m_pLine[0]->GetDistance();
		if (m_pLine[0]->GetDirection() >= 0)
			vect.SetVector(1.0, -90 - m_pLine[0]->GetDirection());
		else vect.SetVector(1.0, 90 - m_pLine[0]->GetDirection());
	}
	if (m_pLine[1]->IsValid() && (dMaxDist < m_pLine[1]->GetDistance())) {	// right line
		dMaxDist = m_pLine[1]->GetDistance();
		if (m_pLine[1]->GetDirection() >= 0)
			vect.SetVector(1.0, 90 - m_pLine[1]->GetDirection());
		else vect.SetVector(1.0, -90 - m_pLine[1]->GetDirection());
	}
	if (m_pLine[2]->IsValid() && (dMaxDist < m_pLine[2]->GetDistance())) {	// top line
		dMaxDist = m_pLine[2]->GetDistance();
		if (m_pLine[2]->GetDirection() >= 0)
			vect.SetVector(1.0, -m_pLine[2]->GetDirection());
		else vect.SetVector(1.0, -180 - m_pLine[2]->GetDirection());
	}
	if (m_pLine[3]->IsValid() && (dMaxDist < m_pLine[3]->GetDistance())) {	// bottom line
		dMaxDist = m_pLine[3]->GetDistance();
		if (m_pLine[3]->GetDirection() >= 0)
			vect.SetVector(1.0, 180 - m_pLine[3]->GetDirection());
		else vect.SetVector(1.0, -m_pLine[3]->GetDirection());
	}
	if (dMaxDist > 0) {
		m_pPlayer[0]->SetBodyDir(vect.GetAngle() - m_pPlayer[0]->GetHeadDir());
		m_pPlayer[0]->SetSpeedVector(m_dSpeed, m_pPlayer[0]->GetBodyDir());
		m_pPlayer[0]->SetRelVector(1.0, vect.GetAngle());

		double sx = 0;
		double sy = 0;
		double n = 0;
		for (i = 0; i < FLAG_COUNT; i++) {
			if (m_pFlag[i]->IsValid()) {
				vect.SetVector(m_pFlag[i]->GetDistance(), m_pPlayer[0]->GetDirection() + m_pFlag[i]->GetDirection());
				p1 = m_pFlag[i]->GetAbsPos() - vect;
				sx += (170 - m_pFlag[i]->GetDistance()) * p1.GetX();
				sy += (170 - m_pFlag[i]->GetDistance()) * p1.GetY();
				n += (170 - m_pFlag[i]->GetDistance());
			}
		}
		m_pPlayer[0]->SetAbsPos(sx / n, sy / n);
		return true;
	}
*/
	// impossible to localize, less than two flags visible
	if (m_iFlagCount < 2) return false;

	// search for two visible flags with maximal difference of the view angle
	for (i = 0; i < m_iFlagCount; i++) {
		tmpAng = m_pFlag[i]->GetDirection();
		if (tmpAng < iMinAngle) {
			iMinAngle = tmpAng;
			iFlag1 = i;
		}
		if (tmpAng > iMaxAngle) {
			iMaxAngle = tmpAng;
			iFlag2 = i;
		}
	}

	// do the calculations
	r1 = m_pFlag[iFlag1]->GetDistance();
	r2 = m_pFlag[iFlag2]->GetDistance();

	p1 = m_pFlag[iFlag1]->GetAbsPos();
	p2 = m_pFlag[iFlag2]->GetAbsPos();

	// calculate the square distance of the two flags
	vect = p2 - p1;
	dist = vect.GetLength();

	r1 *= r1;
	r2 *= r2;
	dist2 = dist * dist;

	// a = (r1^2 - r2^2 + d^2 ) / (2 d)
	a = (r1 - r2 + dist2) / (2.0 * dist);
	// h^2 = r1^2 - a^2
	h = r1 - a*a;
	if (h < 0.0) h = 0.0;
	else h = sqrt(h);

	// P3 = P1 + a ( P2 - P1 ) / d
	vect.SetLength(a);
	p3 = p1 + vect;
	vect.SetLength(a);
	vect.Rotate(90.0);
	vect.SetLength(h);
	m_pPlayer[0]->SetAbsPos(p3 += vect);

	double dDir = 0;
	for (i = 0; i < m_iFlagCount; i++) {
		vect = m_pFlag[i]->GetAbsPos() - p3;
		vect.Rotate(-m_pFlag[i]->GetDirection());
		dDir += vect.GetAngle();
	}

	m_pPlayer[0]->SetBodyDir(dDir / i - m_pPlayer[0]->GetHeadDir());
	m_pPlayer[0]->SetSpeedVector(m_dSpeed, m_pPlayer[0]->GetBodyDir());
	m_pPlayer[0]->SetRelVector(1.0, vect.GetAngle());
//	vect = p1 - p3;
//	vect.Rotate(-m_pFlag[iFlag1]->GetDirection());
//	m_pPlayer[0]->SetBodyDir(vect.GetAngle() - m_pPlayer[0]->GetHeadDir());
//	m_pPlayer[0]->SetSpeedVector(m_dSpeed, m_pPlayer[0]->GetBodyDir());
//	m_pPlayer[0]->SetRelVector(1.0, vect.GetAngle());

	return true;
}

bool CPerception::LocalizeAll()
{
	int i;
	double dDir = m_pPlayer[0]->GetBodyDir() + m_pPlayer[0]->GetHeadDir();

	for (i = 0; i < m_iPlayerCount; i++) {
		m_pTmpPlayer[i]->SetDirection(m_pTmpPlayer[i]->GetDirection() + dDir);
		m_pTmpPlayer[i]->CalcAbsPos(m_pPlayer[0]->GetAbsPos());
/*		if (m_pTmpPlayer[i]->IsChngValid())*/ m_pTmpPlayer[i]->CalcSpeed(m_pPlayer[0]->GetFuturePos(1));
//		if (m_pTmpPlayer[i]->IsDirValid()) {
			m_pTmpPlayer[i]->SetBodyDir(m_pPlayer[0]->GetBodyDir() + m_pTmpPlayer[i]->GetBodyDir());
			m_pTmpPlayer[i]->SetHeadDir(dDir - m_pTmpPlayer[i]->GetBodyDir() + m_pTmpPlayer[i]->GetHeadDir());
//		}
		StorePlayer(*m_pTmpPlayer[i]);
	}
	if (m_pBall->IsLocked()) {
		m_pBall->SetDirection(m_pBall->GetDirection() + dDir);
		m_pBall->CalcAbsPos(m_pPlayer[0]->GetAbsPos());
/*		if (m_pBall->IsChngValid())*/ m_pBall->CalcSpeed(m_pPlayer[0]->GetFuturePos(1));
	}
	if (m_pGoal[0]->IsValid()) m_pGoal[0]->SetDirection(m_pGoal[0]->GetDirection() + dDir);
	if (m_pGoal[1]->IsValid()) m_pGoal[1]->SetDirection(m_pGoal[1]->GetDirection() + dDir);
	for (i = 0; i < m_iFlagCount; i++) {
		if (m_pFlag[i]->IsValid()) m_pFlag[i]->SetDirection(m_pFlag[i]->GetDirection() + dDir);
	}

	return true;
}
/*
// simple method of storing players
void CPerception::StorePlayer(CPlayer& player)
{
	int i, j, k;
	int index;

	if (player.GetSide() == m_pPlayer[0]->GetSide()) {	// player is ours
		j = 1;
		k = PLAYER_COUNT / 2;
	}
	else {						// player is opponent
		j = PLAYER_COUNT / 2;
		k = PLAYER_COUNT;
	}

	// store the player of known side (both previous cases)
	if (player.GetSide() != SIDE_UNKNOWN) {	// the player is known (the side at least)

		for (i = j; i < k; i++) {	// find invalid player to replace
			if (!m_pPlayer[i]->IsValid()) {
				*m_pPlayer[i] = player;
				return;
			}
		}
		int age = 0;
		for (i = j; i < k; i++) {	// find oldest player to replace
			if (m_pPlayer[i]->GetAge() > age) {
				age = m_pPlayer[i]->GetAge();
				index = i;
			}
		}
		*m_pPlayer[index] = player;
	}
	else {						// the player is unknown

		for (i = 0; i < PLAYER_COUNT - 1; i++) {	// find invalid player to replace
			if (!m_pUnknownPlayer[i]->IsValid()) {
				*m_pUnknownPlayer[i] = player;
				return;
			}
		}
		int age = 0;
		for (i = 0; i < PLAYER_COUNT - 1; i++) {	// find oldest player to replace
			if (m_pUnknownPlayer[i]->GetAge() > age) {
				age = m_pUnknownPlayer[i]->GetAge();
				index = i;
			}
		}
		*m_pUnknownPlayer[index] = player;
	}
}
*/

// heuristic method of storing players
void CPerception::StorePlayer(CPlayer& player)
{
	int i, j, k;
	double dDist;
	int index;

	if (player.GetSide() == m_pPlayer[0]->GetSide()) {	// player is ours
		j = 1;
		k = PLAYER_COUNT / 2;
	}
	else {						// player is opponent
		j = PLAYER_COUNT / 2;
		k = PLAYER_COUNT;
	}

	if (player.GetNumber() > NUMBER_UNKNOWN) {	// we know both player's side and number

		// searching exactly the same player
		for (i = j; i < k; i++) {
			if (m_pPlayer[i]->IsValid() && !m_pPlayer[i]->IsLocked() &&
				m_pPlayer[i]->GetNumber() == player.GetNumber()) break;
		}
		if (i < k) {			// we have found the same player number
			*m_pPlayer[i] = player;
			return;
		}

		// searching among unnumbered correct side players
		dDist = 2 * player.GetMaxAge();
		index = -1;
		for (i = j; i < k; i++) {
			if (m_pPlayer[i]->IsValid() && !m_pPlayer[i]->IsLocked() &&
				m_pPlayer[i]->GetNumber() == NUMBER_UNKNOWN)
			{					// get the nearest unnumbered player of the correct side
				double tmpDist = (m_pPlayer[i]->GetAbsPos() - player.GetAbsPos()).GetLength();
				if (dDist > tmpDist && tmpDist < (double)m_pPlayer[i]->GetAge()) {
					dDist = tmpDist;
					index = i;
				}
			}
		}
		// tolerance is one meter for one sim cycle
		if (index > -1) {		// the player is within tolerance
			*m_pPlayer[index] = player;
			return;
		}
	}
	else if (player.GetSide() != SIDE_UNKNOWN) {	// we know only player's side

		// searching among correct side players
		dDist = 2 * player.GetMaxAge();
		index = -1;
		for (i = j; i < k; i++) {
			if (m_pPlayer[i]->IsValid() && !m_pPlayer[i]->IsLocked())
			{					// get the nearest player of the correct side
				double tmpDist = (m_pPlayer[i]->GetAbsPos() - player.GetAbsPos()).GetLength();
				if (dDist > tmpDist && tmpDist < (double)m_pPlayer[i]->GetAge()) {
					dDist = tmpDist;
					index = i;
				}
			}
		}
		// tolerance is one meter for one sim cycle
		if (index > -1) {		// the player is within tolerance
//			player.UpdateState(*m_pPlayer[index]);
			*m_pPlayer[index] = player;
			return;
		}
	}

	// store the player of known side (both previous cases)
	if (player.GetSide() != SIDE_UNKNOWN) {	// we know at least player's side
/*
		// searching among unknown players
		dDist = 2 * player.GetMaxAge();
		index = -1;
		for (i = 0; i < PLAYER_COUNT - 1; i++) {
			if (m_pUnknownPlayer[i]->IsValid() && !m_pUnknownPlayer[i]->IsRecent())
			{					// get the nearest unknown player
				double tmpDist = (m_pUnknownPlayer[i]->GetAbsPos() - player.GetAbsPos()).GetLength();
				if (dDist > tmpDist && tmpDist < (double)m_pUnknownPlayer[i]->GetAge()) {
					dDist = tmpDist;
					index = i;
				}
			}
		}
		// tolerance is one meter for one sim cycle
		if (index > -1) {		// the player is within tolerance
			m_pUnknownPlayer[index]->Invalidate();
		}
*/
		for (i = j; i < k; i++) {	// find invalid player to replace
			if (!m_pPlayer[i]->IsValid()) {
				*m_pPlayer[i] = player;
				return;
			}
		}
		int age = -1;
		for (i = j; i < k; i++) {	// find oldest player to replace
			if (m_pPlayer[i]->GetAge() > age) {
				age = m_pPlayer[i]->GetAge();
				index = i;
			}
		}
		if (index > -1) *m_pPlayer[index] = player;	// condition should be true at all times
	}
	else {						// we don't know anything about the player

		// searching among unknown players
		dDist = 2 * player.GetMaxAge();
		index = -1;
		for (i = 0; i < PLAYER_COUNT - 1; i++) {
			if (m_pUnknownPlayer[i]->IsValid() && !m_pUnknownPlayer[i]->IsLocked())
			{					// get the nearest unknown player
				double tmpDist = (m_pUnknownPlayer[i]->GetAbsPos() - player.GetAbsPos()).GetLength();
				if (dDist > tmpDist && tmpDist < (double)m_pUnknownPlayer[i]->GetAge()) {
					dDist = tmpDist;
					index = i + PLAYER_COUNT;
				}
			}
		}

		// searching among known players
		for (i = 1; i < PLAYER_COUNT; i++) {
			if (m_pPlayer[i]->IsValid() && !m_pPlayer[i]->IsLocked())
			{					// get the nearest known player
				double tmpDist = (m_pPlayer[i]->GetAbsPos() - player.GetAbsPos()).GetLength();
				if (dDist > tmpDist && tmpDist < (double)m_pPlayer[i]->GetAge()) {
					dDist = tmpDist;
					index = i;
				}
			}
		}

		if (index > PLAYER_COUNT) {	// the player is unknown
//			player.UpdateState(*m_pUnknownPlayer[index - PLAYER_COUNT]);
			*m_pUnknownPlayer[index - PLAYER_COUNT] = player;
			return;
		}
		else if (index > 0) {	// the player is known
//			player.UpdateState(*m_pPlayer[index]);
			*m_pPlayer[index] = player;
			return;
		}

		for (i = 0; i < PLAYER_COUNT - 1; i++) {	// find invalid player to replace
			if (!m_pUnknownPlayer[i]->IsValid()) {
				*m_pUnknownPlayer[i] = player;
				return;
			}
		}
		int age = -1;
		for (i = 0; i < PLAYER_COUNT - 1; i++) {	// find oldest player to replace
			if (m_pUnknownPlayer[i]->GetAge() > age) {
				age = m_pUnknownPlayer[i]->GetAge();
				index = i;
			}
		}
		if (index > -1) *m_pUnknownPlayer[index] = player;	// condition should be true at all times
	}
}

///////////////////////////////////////////////////////////////////////////////
// CPerception implementation

CPerception::~CPerception()
{
	int	i;

	// delete visible objects
	for (i = 0; i < PLAYER_COUNT; i++) delete m_pPlayer[i];
	for (i = 0; i < PLAYER_COUNT - 1; i++) {
		delete m_pUnknownPlayer[i];
		delete m_pTmpPlayer[i];
	}

	delete m_pBall;
	delete m_pGoal[0];
	delete m_pGoal[1];

	// delete the flags
	for (i = 0; i < FLAG_COUNT; i++) delete m_pFlag[i];

	// delete the lines
	for (i = 0; i < 4; i++) delete m_pLine[i];
}

double CPerception::EvaluateBallDistance(double power)
{
			double dArg1 = 1 - 0.25 * (m_pBall->GetDirection() - m_pPlayer[0]->GetBodyDir()) / 180 - 0.25 * (m_pBall->GetDistance() - PLAYER_SIZE - BALL_SIZE) / KICKABLE_MARGIN;
			double speed = m_pBall->GetSpeedDist() + dArg1 * KICK_POWER_RATE * power;
      double dist = m_pBall->GetDistance();
      for (int i = 0; i < EVAL_MAX_ITERATIONS; i++) {
        dist += speed;
        speed *= BALL_DECAY;
        }
      return dist;      
}

double CPerception::EvaluatePower(double distance)
{
  double lastDist = 100000.0;
  double newDist;

  for(int i = 100; i >= 0; i -= EVAL_POWER_STEP) {
    newDist = EvaluateBallDistance(i);
      if (newDist < distance) break; 
    lastDist = newDist;
    }
  if (i < 0) {
    return 0;
    }
  return (distance - newDist > lastDist - distance) ? i + EVAL_POWER_STEP : i; 
}

CPlayer* CPerception::GetNearestOpponent(CPlayer* pPlayer, double direction, double tolerance)
{
	CPlayer* pNearestPlayer = NULL;
	double dDist;
	double dMinDist = 1000;

	if (pPlayer == NULL) pPlayer = m_pPlayer[0];
	for (int i = PLAYER_COUNT / 2; i < PLAYER_COUNT; i++) if (m_pPlayer[i]->IsValid() && (m_pPlayer[i] != pPlayer)) {
    CVector dir1(1, pPlayer->GetAngleTo(m_pPlayer[i]));
    dir1.Rotate(-direction);
    dir1.Rotate(-tolerance);
    CVector dir2(1, pPlayer->GetAngleTo(m_pPlayer[i]));
    dir2.Rotate(-direction);
    dir2.Rotate(-direction);
    dir2.Rotate(+tolerance);
    if (direction < dir1.GetAngle() || direction > dir2.GetAngle()) continue;
    dDist = pPlayer->GetDistanceTo(m_pPlayer[i]);
		if (dDist < dMinDist) {
			pNearestPlayer = m_pPlayer[i];
			dMinDist = dDist;
		}
	}

	return pNearestPlayer;

}

double CPerception::GetFreeDirection(double lAngle, double rAngle) {
  double newdir = (lAngle + rAngle) / 2;
  CPlayer *pPlayer = m_pPlayer[0];
  double dirs[PLAYER_COUNT+3];
  double dirs2[PLAYER_COUNT+3][2];
  int dircount = 2;
  dirs[0] = lAngle;
  dirs[1] = rAngle;

  for (int i = 11; i < PLAYER_COUNT; i++) if (m_pPlayer[i]->IsValid()) {
		CPosition pos = m_pPlayer[i]->GetAbsPos();
  	double dAng0 = pPlayer->GetAngleTo(pos);
	  double dDist0 = pPlayer->GetDistanceTo(pos);

    if (dDist0 > 40 || dAng0 >= lAngle || dAng0 <= rAngle) {
      continue;
    }
    for (int j=0; j<dircount; j++) {
      if (dAng0 > dirs[j]) {
        for (int k=dircount; k>j; k--) {
          dirs[k] = dirs[k-1];
        }
        dirs[j] = dAng0;
        dircount++;
        break;
      }
    }
  }

  for (int j=1; j<dircount; j++) {
    dirs2[j-1][0] = (dirs[j] + dirs[j-1]) / 2;
    dirs2[j-1][1] = dirs[j-1] - dirs[j];
  }
  
  dircount--;

  //zistim uhol, kedy mam maximalny priestor
  int maxSpace = 0;
  for (j=1; j<dircount; j++) {
    if (dirs2[j][1] > dirs2[maxSpace][1]) {
      maxSpace = j;
    }
  };

  return dirs2[maxSpace][0];
}
