// Position.h: interface for the CPosition class
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __AFX_POSITION_H_
#define __AFX_POSITION_H_

#if _MSC_VER > 1000
#pragma once
#endif	// _MSC_VER > 1000

// Position.h header file declares following classes

class CPosition;

#include "Vector.h"

///////////////////////////////////////////////////////////////////////////////
// Class: CPosition
// Date: 12.04.01
// Version Date: 12.04.01
// Version: 1.00
// Author: Marek Lehuta
//
// Base Class: CObject
///////////////////////////////////////////////////////////////////////////////
// Description: represents ortogonal position in coordinate system
//				calculates ortogonal position behaviour
///////////////////////////////////////////////////////////////////////////////

class CPosition : public CObject
{
// Construction
public:
	CPosition() { m_dX = m_dY = 0; };
	CPosition(double dX, double dY) { m_dX = dX; m_dY = dY; };
	CPosition(const CPosition& pos) { m_dX = pos.GetX(); m_dY = pos.GetY(); };

// Attributes
private:
	double m_dX;
	double m_dY;

// Operations
public:
	void MoveX(double dX) { m_dX += dX; };
	void MoveY(double dY) { m_dY += dY; };
	void MoveXY(double dX, double dY) { m_dX += dX; m_dY += dY; };

	void SetX(double dX) { m_dX = dX; };
	void SetY(double dY) { m_dY = dY; };
	void SetXY(double dX, double dY) { m_dX = dX; m_dY = dY; };
	double GetX() const { return m_dX; };
	double GetY() const { return m_dY; };

	CPosition operator+(const CVector& vect) const;
	CPosition operator-(const CVector& vect) const;
	CVector operator-(const CPosition& pos) const;

	bool operator==(const CPosition& pos) const;

	const CPosition& operator=(const CPosition& pos);

	const CPosition& operator+=(const CVector& vect);
	const CPosition& operator-=(const CVector& vect);

// Implementation
public:
	virtual ~CPosition() {};

};

///////////////////////////////////////////////////////////////////////////////

#endif	// __AFX_POSITION_H_
