// Vector.cpp: implementation of the CVector class
//
///////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Goalie.h"
#include "Vector.h"

#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif	// _DEBUG

///////////////////////////////////////////////////////////////////////////////
// CVector construction

CVector::CVector(double dLen, double dAng)
{
	m_dLen = dLen;
	m_dAng = dAng;
	AdjustXY();
}

CVector::CVector(const CVector &vect)
{
	m_dLen = vect.GetLength();
	m_dAng = vect.GetAngle();
	m_dX = vect.GetX();
	m_dY = vect.GetY();
}

///////////////////////////////////////////////////////////////////////////////
// CVector operations

void CVector::ScaleXY(double dScX, double dScY)
{
	m_dX *= dScX;
	m_dY *= dScY;
	AdjustAng();
}

void CVector::SetVector(double dLen, double dAng)
{
	m_dLen = dLen;
	m_dAng = dAng;
	AdjustXY();
}

CVector CVector::operator+(const CVector& vect) const
{
	return CVector(m_dX + vect.GetX(), m_dY + vect.GetY(), 0);
}

CPosition CVector::operator+(const CPosition& pos) const
{
	return CPosition(m_dX + pos.GetX(), m_dY + pos.GetY());
}

CVector CVector::operator-(const CVector& vect) const
{
	return CVector(m_dX - vect.GetX(), m_dY - vect.GetY(), 0);
}

CVector CVector::operator*(double dMag) const
{
	return CVector(m_dLen * dMag, m_dAng);
}

CVector CVector::operator/(double dRed) const
{
	return CVector(m_dLen / dRed, m_dAng);
}

CVector CVector::operator+(double dAng) const
{
	return CVector(m_dLen, m_dAng + dAng);
}

CVector CVector::operator-(double dAng) const
{
	return CVector(m_dLen, m_dAng - dAng);
}

bool CVector::operator==(const CVector& vect) const
{
	return (m_dX == vect.GetX() && m_dY == vect.GetY());
}

const CVector& CVector::operator=(const CVector& vect)
{
	m_dLen = vect.GetLength();
	m_dAng = vect.GetAngle();
	m_dX = vect.GetX();
	m_dY = vect.GetY();
	return *this;
}

const CVector& CVector::operator+=(const CVector& vect)
{
	m_dX += vect.GetX();
	m_dY += vect.GetY();
	AdjustAng();
	return *this;
}

const CVector& CVector::operator-=(const CVector& vect)
{
	m_dX -= vect.GetX();
	m_dY -= vect.GetY();
	AdjustAng();
	return *this;
}

void CVector::AdjustAng()
{
	if (m_dX == 0) {
		m_dLen = m_dY;
		if (m_dY < 0) {
			m_dLen = -m_dLen;
			m_dAng = -90;
		}
		else m_dAng = 90;
	}
	else if (m_dY == 0) {
		m_dLen = m_dX;
		if (m_dX < 0) {
			m_dLen = -m_dLen;
			m_dAng = 180;
		}
		else m_dAng = 0;
	}
	else {
		m_dLen = sqrt(m_dX * m_dX + m_dY * m_dY);
		m_dAng = Rad2Deg(atan(m_dY / m_dX));
		if (m_dX < 0 && m_dY < 0) m_dAng -= 180;
		else if (m_dX < 0 && m_dY > 0) m_dAng += 180;
	}
	if (m_dLen == 0) m_dAng = 0;
}

void CVector::AdjustXY()
{
	if (m_dLen == 0) {
		m_dAng = m_dX = m_dY = 0;
		return;
	}
	else if (m_dLen < 0) {
		m_dLen = -m_dLen;
		m_dAng += 180;
	}
	while (m_dAng > 180) m_dAng -= 360;
	while (m_dAng < -180) m_dAng += 360;
	m_dX = m_dLen * cos(Deg2Rad(m_dAng));
	m_dY = m_dLen * sin(Deg2Rad(m_dAng));
}
