// Vector.h: interface for the CVector class
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __AFX_VECTOR_H_
#define __AFX_VECTOR_H_

#if _MSC_VER > 1000
#pragma once
#endif	// _MSC_VER > 1000

// Vector.h header file declares following classes

class CVector;

#include "Position.h"

#define PI			3.1415926535898
#define RAD2DEG		180.0 / PI
#define DEG2RAD		PI / 180.0
#define Rad2Deg(a)	a * RAD2DEG
#define Deg2Rad(a)	a * DEG2RAD

///////////////////////////////////////////////////////////////////////////////
// Class: CVector
// Date: 12.04.01
// Verion Date: 20.04.01
// Version: 1.02
// Author: Marek Lehuta
//
// Base Class: CObject
///////////////////////////////////////////////////////////////////////////////
// Description: represents ortogonal vector in coordinate system
//				calculates ortogonal and angular vector behaviour
///////////////////////////////////////////////////////////////////////////////

class CVector : public CObject
{
// Construction
public:
	CVector() { m_dLen = m_dAng = m_dX = m_dY = 0.0; };
	CVector(double dLen, double dAng);
	CVector(double dX, double dY, int) { m_dX = dX; m_dY = dY; AdjustAng(); };
	CVector(const CVector &vect);

// Attributes
private:
	double m_dLen, m_dAng;
	double m_dX, m_dY;

// Operations
public:
	void Stretch(double dStr) { m_dLen += dStr; AdjustXY(); };
	void Magnify(double dMag) { m_dLen *= dMag; AdjustXY(); };
	void Rotate(double dAng) { m_dAng += dAng; AdjustXY(); };
	void MoveX(double dX) { m_dX += dX; AdjustAng(); };
	void MoveY(double dY) { m_dY += dY; AdjustAng(); };
	void MoveXY(double dX, double dY) { m_dX += dX; m_dY += dY; AdjustAng(); };
	void ScaleX(double dScX) { m_dX *= dScX; AdjustAng(); };
	void ScaleY(double dScY) { m_dY *= dScY; AdjustAng(); };
	void ScaleXY(double dScX, double dScY);

	void SetLength(double dLen) { m_dLen = dLen; AdjustXY(); };
	void SetAngle(double dAng) { m_dAng = dAng; AdjustXY(); };
	void SetVector(double dLen, double dAng);
	void SetX(double dX) { m_dX = dX; AdjustAng(); };
	void SetY(double dY) { m_dY = dY; AdjustAng(); };
	void SetXY(double dX, double dY) { m_dX = dX; m_dY = dY; AdjustAng(); };
	double GetLength() const { return m_dLen; };
	double GetAngle() const { return m_dAng; };
	double GetX() const { return m_dX; };
	double GetY() const { return m_dY; };

	CVector operator+(const CVector& vect) const;
	CPosition operator+(const CPosition& pos) const;
	CVector operator-(const CVector& vect) const;

	CVector operator*(double dMag) const;
	CVector operator/(double dRed) const;
	CVector operator+(double dAng) const;
	CVector operator-(double dAng) const;

	bool operator==(const CVector& vect) const;

	const CVector& operator=(const CVector& vect);

	const CVector& operator+=(const CVector& vect);
	const CVector& operator-=(const CVector& vect);

private:
	void AdjustAng();
	void AdjustXY();

// Implementation
public:
	virtual ~CVector() {};

};

///////////////////////////////////////////////////////////////////////////////

#endif	// __AFX_VECTOR_H_
